macroScript charRigger
	category:"Comet Cartoons"
	toolTip:"charRigger: Given named pointHelpers, this creates an IK character rig."
	Icon:#("CometCartoons",14)
(
/*
 * charCreator.mcr -  Copyright 2001 Michael B. Comet  All Rights Reserved.
 *
 *  Version 1.00 - Michael B. Comet - comet@comet-cartoons.com - http://www.comet-cartoons.com/
 *					10/07/01 - Initial Release Rev.
 *
 *	Version 1.01 - comet@comet-cartoons.com 10/08/01
 *			Added bone color mixing
 *
 *  Version 1.02 -comet@comet-cartoons.com 10/09/01
 *			Fixed "connect Required SubAnim" bug fix...Thanks to "Lil Dragon" for helping!
 *
 *  Version 1.03 -comet@comet-cartoons.com 10/10/01
 *			Fixed error where Right Hand Bone is aligned wrong, Thanks to 3dmastering for the help.
 *			Added WristTwist bones and attribute.
 *			Fixed up other controls and bones not being Euler by default.
 *			Made characters with spaces in prefix convert to _ for naming.
 *			Feet and Hands now have 3Dish controls for easier grabbing.
 *			Finger curves a bit smaller
 *
 *  Version 1.04 -comet@comet-cartoons.com 10/10/01
 *			Made undo stuff have a nice label for the undo menu.
 *			Set up global Version and Build strings for about box and title.
 *
 *	Version 1.05 -comet@comet-cartoons.com 10/11/01
 *			Bones and Points handle small scale character now.
 *			Fixed feet to read angle from ptAnkle so the don't have to be straight.
 *			Fixed spine creation so things aren't rotated wrong due to parenting
 *				technique.  It now parents controls to controls...so axis match.
 *			Placement size calculation improved.
 *			Fixed Head and Neck creation so things aren't rotated wrong due to
 *				parenting technique.  Control axis will now match, like spine.
 *				Neck also properly resets controls in case you run it a 2nd time
 *				on a rig without neck1 or neck2
 *			Fixed arm and Finger bone creation so sizes are right.
 *			Fixed clav so shrug is always up.
 *
 *	Version 1.06 - comet@comet-cartoons.com 10/12/01
 *			Changed selection Set names.
 *
 *  Version 1.07 - comet@comet-cartoons.com 11/10/01
 *			Made sure hip is parented to placement on hip and place create
 *			Added new stuff to Mirror ptHelpers left or right with proper names
 *			Made arm "World" helpers now a ctrl that is visible and with a 
 *				LinkController on it, so that it can easily be linked to
 *				different objects at different times during animation for 
 *				external hand locking.
 *
 * Version 1.08 - comet@comet-cartoons.com 11/26/01
 *			Fixed some bugs in finger creation that occured if you tried to create
 *			only 1 or 0 fingers.
 *
 * Version 1.09 - comet@comet-cartoons.com 01/27/02
 *			Made the toe tip bone at proper angle for feet now.
 *			Also made end effectors at proper rotation too so it looks
 *			prettier.
 *
 * Version 1.10 - comet@comet-cartoons.com 01/27/02
 *			Fixed bug of creating foot after ground and placement existed would give error.
 *			Fixed footCTRL and roll rotation issues, where in some cases footRoll and
 *				the footCTRL would be at a skewed angle to what it should have been.
 *				Feet will now properly be in alignment the the ptAnkle helpers.
 *
 * Version 1.11 - comet@comet-cartoons.com 01/27/02
 *			Started to add Toe creation support in.  Currently will properly create toe
 *				attributess on the objects as needed.  From the addToeAttrs() proc.
 * 			Also rearranged UI slightly.  Eventually I will hopefully add true toe 
 *			creation in.
 *
 * Version 1.20 - comet@comet-cartoons.com 03/03/02
 *			Added toe support.  You can add up 5 toes with 1-3 digits each like fingers.
 *			The first toe is optional, and can be rotated as desired, like a thumb, (good
 *			for monkey characters).   The other toes can be rotated in top view.
 *			Note: these are smaller "toes" vs. the master toe bone that gets create regardless
 *			when you create a foot.  It's only needed if your character really has separate 
 *			toe finger-like detail.
 *
 * Version 1.21 - comet@comet-cartoons.com 03/03/02
 *			Also added new Digit multipliers to the finger and toes.  This allows digits farther 
 *			down the chain to rotate more or less.  This gives a more natural curl.  By default
 *			the last finger digit will rotate more than the others, giving a more natural look,
 *			will not cause the fingers to dig into the hand at 100% rotation now.
 *
 * Version 1.22 - comet@comet-cartoons.com 03/04/02
 *			Added zeroOut function in based on my zeroOut script.  This now lets all 
 *			basic controls get "zero" for PRS data, so that the rig can be reset in the
 *			motion panel, or via parent space viewing, or via the new attrKey, to 0,0,0
 *			which will put everything back to its default pose.
 *
 *			Also fixed up heelINT dummy position for when foot is raised
 *			so that footRoll still goes backwards properly.
 *
 * Version 1.23 - comet@comet-cartoons.com 03/04/02
 *			Fixed bug in createToes and createFingers where if no finger/toe helpers
 *			existed, it would throw an error since it was trying to create attributes
 *			for nothing...
 *			Also made UI a little more compacted/smaller
 *
 * Version 1.24 - comet@comet-cartoons.com 03/04/02
 *			Made toe end effector in Start Parent Space so it doesn't wiggle at an angles.
 *
 * Version 1.25 - comet@comet-cartoons.com 03/09/02
 *			Fixed buttons in UI for L/R being reversed for legs and arms.
 *
 * Version 1.30 - comet@comet-cartoons.com 03/09/02
 *			Added support for quadrapeds by allowing a second pair of Legs.  These are named 
 *			with ptHelpers the same, except L and R becomes L2 and R2.  ie: ptL2Leg and ptR2Leg etc...
 *			By default, the front legs start dummy is parented to the last spine control created, or
 *			else the hips.  Note you can still have arms and hands too.  ie: like a centaur with 
 *			4 legs and 2 arms...see the sample files.
 *			If you want more than 4 legs, you can probably hack this code by changing the calls for
 *				the ui buttons to do "L3" etc...and then changing the createLeg script to handle
 *				those prefixes wherever you see a LR if statement.
 *			You can now have up to 6 spine bones.
 *
 * Version 1.31 - comet@comet-cartoons.com 03/10/02
 *			Added support for up to 8 neck ctrls/bones and helpers.
 *			Neck, Head and Spine Controls are now created at the proper orientation,
 *				instead of just flat.  This is mostly a cosmetic thing...but makes it
 *				nicer.  They are still "zeroed" out properly for use with attrKey etc...
 *
 *
 * List of Required Locators.
 *		ptHips		- Hip
 *
 *			(Legs are optional in that you can just have one, or none at all)
 *			(!!!Local Z angle of ptAnkle (L&R) in top view controls angle of
 *				foot bone creation from top view, so the feet don't have to
 *				point straight forward.!!!)
 *		ptLLeg		- L Upper Leg Start	
 *		ptLKnee		- Knee pos, where lower leg starts
 *		ptLAnkle	- Ankle of foot
 *		ptLBall		- Ball position on foot, where toes start
 *		ptLToe		- Toe TIP on foot.
 *
 *		ptRLeg		- L Upper Leg Start
 *		ptRKnee		- Knee pos, where lower leg starts
 *		ptRAnkle	- Ankle of foot
 *		ptRBall		- Ball position on foot, where toes start
 *		ptRToe		- Toe TIP on foot.
 *
 *		ptL2Leg		- L Upper Leg Start			-- for FRONT Legs stuff for quadraped!  change L to L2 basically.
 *		ptL2Knee		- Knee pos, where lower leg starts
 *		ptL2Ankle	- Ankle of foot
 *		ptL2Ball		- Ball position on foot, where toes start
 *		ptL2Toe		- Toe TIP on foot.
 *
 *		ptR2Leg		- L Upper Leg Start			-- for FRONT Legs stuff for quadraped!  change R to R2 basically.
 *		ptR2Knee		- Knee pos, where lower leg starts
 *		ptR2Ankle	- Ankle of foot
 *		ptR2Ball		- Ball position on foot, where toes start
 *		ptR2Toe		- Toe TIP on foot.
 *
 *		ptS1		- Spine 1 Start		-- optional
 *		ptS2		- Spine 2 Start		-- optional
 *		ptS3		- Spine 3 Start		-- optional
 *		ptS4		- Spine End Top		-- optional
 *		ptS5		- Spine End Top		-- optional
 *		ptS6		- Spine End Top		-- optional
 *		ptS7		- Spine End Top		-- optional
 *
 *		ptNeck1		- Neck ctrl 1		-- optional
 *		ptNeck2		- Neck ctrl 2		-- optional
 *		ptNeck1		- Neck ctrl 3		-- optional
 *		ptNeck2		- Neck ctrl 4		-- optional
 *		ptNeck1		- Neck ctrl 5		-- optional
 *		ptNeck2		- Neck ctrl 6		-- optional
 *		ptNeck1		- Neck ctrl 7		-- optional
 *		ptNeck2		- Neck ctrl 8		-- optional
 *		ptHead		- Head start
 *		ptHeadTop	- Head end
 *
 *		ptLClav		- Clavicle
 *		ptLUpArm	- Upper Arm
 *		ptLLoArm	- Lower Arm
 *		ptLHand		- Hand Start
 *		ptLHandTip	- End of Hand
 *
 *		ptRClav		- Clavicle
 *		ptRUpArm	- Upper Arm
 *		ptRLoArm	- Lower Arm
 *		ptRHand		- Hand Start
 *		ptRHandTip	- End of Hand
 *
 *		ptThumbL1	- Thumb digits, 1st 2 must exist for thumb to be created
 *		ptThumbL2
 *		ptThumbL3
 *		ptThumbL4
 *		ptThumbLUp	- Must exist for thumb.  Represents "up" direction for thumb
 *		ptFinger1L1	- Finger 1 digits
 *		ptFinger1L2
 *		ptFinger1L3
 *		ptFinger1L4
 *		ptFinger2L1	- Finger 2 digits
 *		ptFinger2L2
 *		ptFinger2L3
 *		ptFinger2L4
 *		ptFinger3L1	- Finger 3 digits
 *		ptFinger3L2
 *		ptFinger3L3
 *		ptFinger3L4
 *		ptFinger4L1	- Finger 4 digits
 *		ptFinger4L2
 *		ptFinger4L3
 *		ptFinger4L4
 *
 *		ptThumbR1	- Thumb digits, 1st 2 must exist for thumb to be created
 *		ptThumbR2
 *		ptThumbR3
 *		ptThumbR4
 *		ptThumbRUp	- Must exist for thumb.  Represents "up" direction for thumb
 *		ptFinger1R1	- Finger 1 digits
 *		ptFinger1R2
 *		ptFinger1R3
 *		ptFinger1R4
 *		ptFinger2R1	- Finger 2 digits
 *		ptFinger2R2
 *		ptFinger2R3
 *		ptFinger2R4
 *		ptFinger3R1	- Finger 3 digits
 *		ptFinger3R2
 *		ptFinger3R3
 *		ptFinger3R4
 *		ptFinger4R1	- Finger 4 digits
 *		ptFinger4R2
 *		ptFinger4R3
 *		ptFinger4R4
 *
 *		ptToe1L1, ptToe1L2, ptToe1L3, ptToe1L4;		-- replace L with R for R toes.
 *		ptToe2L1, ptToe2L2, ptToe2L3, ptToe2L4;
 *		ptToe3L1, ptToe3L2, ptToe3L3, ptToe3L4;
 *		ptToe4L1, ptToe4L2, ptToe4L3, ptToe4L4;
 *		ptToe5L1, ptToe5L2, ptToe5L3, ptToe5L4;
 *		ptToeLUp;									-- up vector for first toe.
 *
 *				-- and then also for FRONT toes... L or R becomes L2 and R2
 *		ptToe1L21, ptToe1L22, ptToe1L23, ptToe1L24;		-- replace L with R for R toes.
 *		ptToe2L21, ptToe2L22, ptToe2L23, ptToe2L24;
 *		ptToe3L21, ptToe3L22, ptToe3L23, ptToe3L24;
 *		ptToe4L21, ptToe4L22, ptToe4L23, ptToe4L24;
 *		ptToe5L21, ptToe5L22, ptToe5L23, ptToe5L24;
 *		ptToeL2Up;									-- up vector for first toe.
 *
 *
 * Wishlist/TODO Items:
 *
 * Bugs/Known Issues:
 *		- One reloading a rigged character, IK end effectors may "appear"
 *			to shoot off screen.  Just refresh the viewport by either
 *			scrubbing back and forth a frame, or by selecting and moving
 *			a related CTRL object.
 */


/*
 * Global Vars
 */
global string cRversion="1.31";
global string cRdate = "Mar 10, 2002";

global string prefix="char";
global bodyColor = color 160 240 60;
global bodyColor2 = color 255 220 0;
global headColor = color 230 100 0;
global baseColor = color 50  220 190;
global leftColor = color 40 80 200;
global rightColor = color 80 200 40;
global internalColor = color 140 20 0;
global boneColor = color 174 186 203;

-- Global CTRL objects that get created
global hipCTRL = undefined;
global boolean stretchyLegs = true;
global boolean headIndep = true;
global boolean stretchyArms = true;
global fDig1Mult = 0.7;
global fDig2Mult = 0.8;
global fDig3Mult = 1.2;
global tDig1Mult = 0.7;
global tDig2Mult = 0.8;
global tDig3Mult = 1.2;



-- This is the range we'll set the timeline too when rigging so that 
--	our expressions and wiring won't die later when animating...
global rStart = -1000;
global rEnd = 10000;

-- Functions -------------------------------------------------------------------------------------

/*
 * objExists() - Given an object name, with or w/o $, this tells if it exists or not.
 *		Returns true on exists, false otherwise.
 */
fn objExists objname =
(
    name="";
	
	if (objname[1] == "$") then		-- already have $ in it?
		name = objname;
	else
		name = ("$"+objname);

	ret = execute(name+" != undefined");
		
	return (ret);
)

-- -----------------------------------------------------------------------------------------------

/*
 * zeroOut - Zero's out an objects PRS data by parenting another object with the suffix ZEROINT 
 *		at the end.  This is a straight grab from my zeroOut macro script...except this takes
 *		ONE object to zero.
 */
fn zeroOut obj =
(
	pt = Point pos:[0,0,0] isSelected:on centermarker:on axistripod:off cross:off Box:off constantscreensize:off drawontop:off size:20;
		-- Insert into hierarchy
	pt.parent = obj.parent;
		-- Now snap it to same values
	coordsys world 
		(
		pt.scale = obj.scale;
		pt.rotation = obj.rotation;
		pt.position = obj.position;
		pt.wirecolor = (color 80 10 0);
		pt.name = (obj.name+"ZEROINT");
		)
		-- reparent object
	obj.parent = pt;
		
)		

-- -----------------------------------------------------------------------------------------------

/*
 * setTimeline() - This will set the timeline to the given start and end times
 */
fn setTimeline start end =
(
	animationRange = (interval start end);
)


-- -----------------------------------------------------------------------------------------------

/* 
 * replaceChar() - Given a string replaces all occurences of character f (from) with 
 *					character t (to)
 */
fn replaceChar str f t =
(
    if (str == "" or str == undefined) then
		return "";
		
	len = str.count;

	for i in 1 to len do
		(
		if (str[i] == f) then	-- replace it needed
			str[i] = t;		
		)

    return str;
)

-- -----------------------------------------------------------------------------------------------



/*
 * boneSetup() - Given a bone, set it up to be right size and color
 *		doFins will set fins on or off
 *		mixColor if provided will cause the regular bone color to mix in with it
 */
fn boneSetup boneObj doFins mixColor:boneColor =
(
    if (boneObj == undefined) then
		return 0;

	-- what is length of bone?
	len = boneObj.length;

	-- how big overall width height?
	wh = len*0.17;
--	if (wh < 5) then
--		wh = 5;

	finSz = len*0.26;
	
	-- make settings happen!		
	boneObj.wirecolor = (0.15*mixColor)+(0.85*boneColor);
	boneObj.width = wh;
	boneObj.height = wh;
	boneObj.sidefins = doFins;
	boneObj.frontfin = doFins;
	boneObj.backfin = doFins;
	boneObj.sidefinssize = finSz;
	boneObj.frontfinsize = finSz;
	boneObj.backfinsize = finSz;
	
	return 1;
)	
-- -----------------------------------------------------------------------------------------------
	
/* 
 *  makeNiceSets() - This will make nice selection sets for all created rig stuff.
 */
fn makeNiceSets =
(
	selectionSets[(prefix+"_CTRLs")] = $*CTRL;
	selectionSets[(prefix+"_BONEs")] = $*BONE;
	selectionSets[(prefix+"_HIDEinternal")] = $*INT;
	selectionSets[(prefix+"_HIDEendEffectors")] = $*EndEffector;
	selectionSets[(prefix+"_DELETEptHelpers")] = $pt*;

	clearSelection();	-- not sets, but actual objects currently selected

		-- And HIDE stuff	
	hide selectionSets[(prefix+"_HIDEendEffectors")];
	hide selectionSets[(prefix+"_HIDEinternal")];
	hide selectionSets[(prefix+"_DELETEptHelpers")];	-- for now don't delete, just hide

    return 1;
)

-- -----------------------------------------------------------------------------------------------

/*
 * makeStretchyBones() - This will make a standard 2 joint stretchy system.
 *	upper, lower, tip are the joints in the stretchy chain in order from parent to child.
 *		tip is required since lower and tip actually have the script on them.
 *  ik   is the ik end effector for the chain
 *	ctrl is the CTRL object that has the maxStretch and extraStretch attributes on it.
 *	dep  is a string of what objects this will depend on for moving
 */
fn makeStretchyBones upper lower tip ik ctrl dep =
(
    if (upper == undefined) then 
		(
		format ("charCreator: makeStretchyBones(), upper is undefined!\n");
		return 0;
		)
    if (lower == undefined) then 
		(
		format ("charCreator: makeStretchyBones(), lower is undefined!\n");
		return 0;
		)
    if (tip == undefined) then 
		(
		format ("charCreator: makeStretchyBones(), tip is undefined!\n");
		return 0;
		)
		
	format "Making Stretchy bone chain\n";
		
		-- First make sure controllers are proper type
    lower.position.controller = Position_XYZ();
    tip.position.controller = Position_XYZ();

		-- store lengths of bones
	upLen = upper.length;
	loLen = lower.length;
	totalLen = upLen + loLen;
    format "Stretchy Bone Lengths: % + % = %.\n" upLen loLen totalLen;

    cntLO = lower.position.controller.x_position.controller = float_script();
    cntTP = tip.position.x_position.controller = float_script();

	if (EmptyModifier != undefined) then 
		ah = "Attribute_Holder.";
	else
		ah = "";

    str = "";
	str += ("dependsOn " +dep+";\n");
	str += ("d = distance $'"+ik.name+"'.pos $"+upper.name+".pos;\n");
	str += ("st = (d - "+(totalLen as string)+");	-- Length of Upper and Lower joints extended\n");
	str += ("orig = "+(upLen as string)+";		-- original bone.length of parent\n");
	str += ("if (st < 0) then	-- cap at 0 since if we \"squashed\" the leg\n");
	str += ("	st = 0;	-- it would never actually bend\n");
	str += ("m = $"+ctrl.name+"."+ah+"maxstretch;\n");
	str += ("if (st > m) then 	-- cap stretch at max val as set by slider\n");
	str += ("	st = m;\n");
	str += ("st = st + $"+ctrl.name+"."+ah+"extrastretch;\n");
	str += ("p = orig + (st/2);	-- divide by 2 since 2 joints in stretch chain\n");
	cntLO.script = str;
	
    str = "";
	str += ("dependsOn " +dep+";\n");
	str += ("d = distance $'"+ik.name+"'.pos $"+upper.name+".pos;\n");
	str += ("st = (d - "+(totalLen as string)+");	-- Length of Upper and Lower joints extended\n");
	str += ("orig = "+(loLen as string)+";		-- original bone.length of parent\n");
	str += ("if (st < 0) then	-- cap at 0 since if we \"squashed\" the leg\n");
	str += ("	st = 0;	-- it would never actually bend\n");
	str += ("m = $"+ctrl.name+"."+ah+"maxstretch;\n");
	str += ("if (st > m) then 	-- cap stretch at max val as set by slider\n");
	str += ("	st = m;\n");
	str += ("st = st + $"+ctrl.name+"."+ah+"extrastretch;\n");
	str += ("p = orig + (st/2);	-- divide by 2 since 2 joints in stretch chain\n");
	cntTP.script = str;
	
    return 1;
)
		-- wrist twist bones stay in the right place when stretchy stuff happens.
-- -----------------------------------------------------------------------------------------------

/*
 * makeStretchyTwist() -  Sets up a special script for twist bones that are 
 *		parented to a regular stretchy bone...
 */
fn makeStretchyTwist num neg twB pB upB ik ctrl dep =
(
    if (twB == undefined) then 
		(
		format ("charCreator: makeStretchyTwist(), twistBone is undefined!\n");
		return 0;
		)
    if (pB == undefined) then 
		(
		format ("charCreator: makeStretchyTwist(), parentBone is undefined!\n");
		return 0;
		)

	format "Making Stretchy Twist for %\n" twB.name;
		
		-- First make sure controllers are proper type
    twB.position.controller = Position_XYZ();

		-- store data for bones
    origPos = (twB.pos.x - pB.pos.x)*neg;
    totalLen = distance ik upB;

    cntTW = twB.position.controller.x_position.controller = float_script();

	if (EmptyModifier != undefined) then 
		ah = "Attribute_Holder.";
	else
		ah = "";

    str = "";
	str += ("dependsOn " +dep+";\n");
	str += ("d = distance $'"+ik.name+"'.pos $"+upB.name+".pos;\n");
	str += ("st = (d - "+(totalLen as string)+");	-- Length of joints extended\n");
	str += ("orig = "+(origPos as string)+";		-- original bone pos\n");
	str += ("if (st < 0) then	-- cap at 0 since if we \"squashed\" the leg\n");
	str += ("	st = 0;	-- it would never actually bend\n");
	str += ("m = $"+ctrl.name+"."+ah+"maxstretch;\n");
	str += ("if (st > m) then 	-- cap stretch at max val as set by slider\n");
	str += ("	st = m;\n");
	str += ("st = st + $"+ctrl.name+"."+ah+"extrastretch;\n");
	str += ("p = orig + (st/2*"+(num as string)+"/4);\n");
	cntTW.script = str;
)

-- -----------------------------------------------------------------------------------------------

/*
 * addFootAttrs() - Adds foot attributes onto the object.
 *    This defines a CA definiton.
 *	  Then it adds some custom data as an array of two sub arrays
 *			the first being an array of MAX's CAT_ParamBlock structs
 *			the second is an array of MAX's CAT_UIItem structs, 
 *				with one item per attribute.  This custom data stores info
 *				about the range, so other stuff, like my attrKey script can use 
 *				and get the range data back out.  
 *	  Finally the data is then actually added onto the object.  If the user has
 *		the EmptyModifer() aka Attribute Holder modifier installed, then it 
 *		will tack that onto the object.  Otherwise it adds it to the base object.
 */
fn addFootAttrs obj =
(
    if (obj == undefined) then 
		(
		format ("charCreator INTERNAL ERROR: addFootAttrs(), obj is undefined!\n");
		return();
		)

		-- Make sure user installed the AttributeHolder modifier.
    if (EmptyModifier != undefined) then 
		(
			-- Add Attr Holder modifier
		addModifier obj (EmptyModifier());
		)

		--
		-- main attr def stuff
		--
    str = "footAttrs = attributes Custom_Attributes\n"+
		"(\n"+
	    "parameters main rollout:footAttrsRO\n"+
		"	(\n"+
		"	roll type:#float default:0.0 ui:rollSpin\n"+
		"	toeRotate type:#float default:0.0 ui:toeRotateSpin;\n";
		if (stretchyLegs) then
			(
			str += "	maxstretch type:#float default:0.0 ui:maxstretchSpin\n"+
			"	extrastretch type:#float default:0.0 ui:extrastretchSpin;\n";
			)		
		str += "	)\n"+
		"   \n"+
		"rollout footAttrsRO \"Foot Anim Parameters\"\n"+ 
		"   (\n"+
		"   spinner 'rollSpin' \"roll\" type:#float range:[-100,100,0];\n"+
		"   spinner 'toeRotateSpin' \"toeRotate\" type:#float range:[-180,180,0];\n";
		if (stretchyLegs) then
			(
			str += "   spinner 'maxstretchSpin' \"maxStretch\" type:#float range:[0,9000,0];\n"+
			"   spinner 'extrastretchSpin' \"extraStretch\" type:#float range:[-9000,9000,0];\n";
			)		
	    str += "   )\n"+
		");\n"+
		"\n" ;
	execute str;

		--
		-- Add some custom def data like MAX does, so we can retrieve RANGE info later for things like my attrKey script.
		--
	str = "\n"+
		"pb1 = CAT_ParamBlock name:\"'roll'\" type:\"Float\" ui:\"rollSpin\" default:\"0.0\";\n"+
		"pb2 = CAT_ParamBlock name:\"'toeRotate'\" type:\"Float\" ui:\"toeRotateSpin\" default:\"0.0\";\n";
		if (stretchyLegs) then
			(
			str += "pb3 = CAT_ParamBlock name:\"'maxstretch'\" type:\"Float\" ui:\"maxstretchSpin\" default:\"0.0\";\n"+
			"pb4 = CAT_ParamBlock name:\"'extrastretch'\" type:\"Float\" ui:\"extrastretchSpin\" default:\"0.0\";\n";
			)		
		str += "ui1 = CAT_UIItem ui:\"Spinner\" name:\"'roll'\" type:\"Float\" range:\"[-100,100,0]\";\n"+ 
		"ui2 = CAT_UIItem ui:\"Spinner\" name:\"'toeRotate'\" type:\"Float\" range:\"[-180,180,0]\";\n";
		if (stretchyLegs) then
			(
			str += "ui3 = CAT_UIItem ui:\"Spinner\" name:\"'maxstretch'\" type:\"Float\" range:\"[0,9000,0]\";\n"+ 
			"ui4 = CAT_UIItem ui:\"Spinner\" name:\"'extrastretch'\" type:\"Float\" range:\"[-9000,9000,0]\";\n";
			)		
		if (stretchyLegs) then
			(
			str += "arr1 = #(pb1, pb2, pb3, pb4);\n"+
			"arr2 = #(ui1, ui2, ui3, ui4);\n";
			)
		else (
			str += "arr1 = #(pb1, pb2);\n"+
			"arr2 = #(ui1, ui2);\n";
			)
		str += "theArr = #(arr1, arr2)\n"+
		"\n"+
		"custAttributes.setDefData footAttrs theArr;\n";
	execute str;
		
    if (EmptyModifier != undefined) then 
		str = "custAttributes.add $"+(obj.name)+".Attribute_Holder footAttrs;";
	else
		str = "custAttributes.add $"+(obj.name)+" footAttrs;";
	execute str;
		-- 
		-- Now add a default controller too
		--	
    if (EmptyModifier != undefined) then 
		(
		str = "";
		str += "$"+(obj.name)+".Attribute_Holder.roll.controller = Bezier_Float();\n";
		str += "$"+(obj.name)+".Attribute_Holder.toeRotate.controller = Bezier_Float();\n";
		)
	else
		(
		str = "";
		str += "$"+(obj.name)+".roll.controller = Bezier_Float();\n";
		str += "$"+(obj.name)+".toeRotate.controller = Bezier_Float();\n";
		)
	execute str;
	
)


-- -----------------------------------------------------------------------------------------------

/*
 * addArmAttrs() - Adds arm attributes onto the object.
 *    This defines a CA definiton.
 *	  Then it adds some custom data as an array of two sub arrays
 *			the first being an array of MAX's CAT_ParamBlock structs
 *			the second is an array of MAX's CAT_UIItem structs, 
 *				with one item per attribute.  This custom data stores info
 *				about the range, so other stuff, like my attrKey script can use 
 *				and get the range data back out.  
 *	  Finally the data is then actually added onto the object.  If the user has
 *		the EmptyModifer() aka Attribute Holder modifier installed, then it 
 *		will tack that onto the object.  Otherwise it adds it to the base object.
 */
fn addArmAttrs obj =
(
    if (obj == undefined) then 
		(
		format ("charCreator INTERNAL ERROR: addArmAttrs(), obj is undefined!\n");
		return();
		)

		-- Make sure user installed the AttributeHolder modifier.
    if (EmptyModifier != undefined) then 
		(
			-- Add Attr Holder modifier
		addModifier obj (EmptyModifier());
		)

		--
		-- main attr def stuff
		--
    str = "armAttrs = attributes ArmCustom_Attributes\n"+
		"(\n"+
	    "parameters main rollout:armAttrsRO\n"+
		"	(\n"+
		"	localworld type:#float default:0.0 ui:localworldSpin\n"+
		"	wristtwist type:#float default:0.0 ui:wristtwistSpin\n"+
		"	reach type:#float default:0.0 ui:reachSpin\n"+
		"	shrug type:#float default:0.0 ui:shrugSpin\n";
		if (stretchyArms) then
			(
			str += "	maxstretch type:#float default:0.0 ui:maxstretchSpin\n"+
			"	extrastretch type:#float default:0.0 ui:extrastretchSpin;\n";
			)		
		str += "	)\n"+
		"   \n"+
		"rollout armAttrsRO \"Arm Anim Parameters\"\n"+ 
		"   (\n"+
		"   spinner 'localworldSpin' \"localWorld\" type:#float range:[0,100,0];\n"+
		"   spinner 'wristtwistSpin' \"wristTwist\" type:#float range:[-720,720,0];\n"+
		"   spinner 'reachSpin' \"reach\" type:#float range:[-100,100,0];\n"+
		"   spinner 'shrugSpin' \"shrug\" type:#float range:[-100,100,0];\n";
		if (stretchyArms) then
			(
			str += "   spinner 'maxstretchSpin' \"maxStretch\" type:#float range:[0,9000,0];\n"+
			"   spinner 'extrastretchSpin' \"extraStretch\" type:#float range:[-9000,9000,0];\n";
			)		
	    str += "   )\n"+
		");\n"+
		"\n" ;
	execute str;

		--
		-- Add some custom def data like MAX does, so we can retrieve RANGE info later for things like my attrKey script.
		--
	str = "\n"+
		"pb1 = CAT_ParamBlock name:\"'localworld'\" type:\"Float\" ui:\"localworldSpin\" default:\"0.0\";\n"+
		"pb1a = CAT_ParamBlock name:\"'wristtwist'\" type:\"Float\" ui:\"wristtwistSpin\" default:\"0.0\";\n"+
		"pb2 = CAT_ParamBlock name:\"'reach'\" type:\"Float\" ui:\"reachSpin\" default:\"0.0\";\n"+
		"pb3 = CAT_ParamBlock name:\"'shrug'\" type:\"Float\" ui:\"shrugSpin\" default:\"0.0\";\n";
		if (stretchyArms) then
			(
			str += "pb4 = CAT_ParamBlock name:\"'maxstretch'\" type:\"Float\" ui:\"maxstretchSpin\" default:\"0.0\";\n"+
			"pb5 = CAT_ParamBlock name:\"'extrastretch'\" type:\"Float\" ui:\"extrastretchSpin\" default:\"0.0\";\n";
			)		
		str += "ui1 = CAT_UIItem ui:\"Spinner\" name:\"'localworld'\" type:\"Float\" range:\"[0,100,0]\";\n"+
		"ui1a = CAT_UIItem ui:\"Spinner\" name:\"'wristtwist'\" type:\"Float\" range:\"[-720,720,0]\";\n"+
		"ui2 = CAT_UIItem ui:\"Spinner\" name:\"'reach'\" type:\"Float\" range:\"[-100,100,0]\";\n"+
		"ui3 = CAT_UIItem ui:\"Spinner\" name:\"'shrug'\" type:\"Float\" range:\"[-100,100,0]\";\n";
		if (stretchyArms) then
			(
			str += "ui4 = CAT_UIItem ui:\"Spinner\" name:\"'maxstretch'\" type:\"Float\" range:\"[0,9000,0]\";\n"+ 
			"ui5 = CAT_UIItem ui:\"Spinner\" name:\"'extrastretch'\" type:\"Float\" range:\"[-9000,9000,0]\";\n";
			)		
		if (stretchyArms) then
			(
			str += "arr1 = #(pb1, pb1a, pb2, pb3, pb4, pb5);\n"+
			"arr2 = #(ui1, ui1a, ui2, ui3, ui4, ui5);\n";
			)
		else (
			str += "arr1 = #(pb1, pb1a, pb2, pb3);\n"+
			"arr2 = #(ui1, ui1a, ui2, ui3);\n";
			)
		str += "theArr = #(arr1, arr2)\n"+
		"\n"+
		"custAttributes.setDefData armAttrs theArr;\n";
	execute str;
		
    if (EmptyModifier != undefined) then 
		str = "custAttributes.add $"+(obj.name)+".Attribute_Holder armAttrs;";
	else
		str = "custAttributes.add $"+(obj.name)+" armAttrs;";
	execute str;
		-- 
		-- Now add a default controller too
		--	
    if (EmptyModifier != undefined) then 
		(
		str = "";
		str += "$"+(obj.name)+".Attribute_Holder.localworld.controller = Bezier_Float();\n";
		str += "$"+(obj.name)+".Attribute_Holder.wristtwist.controller = Bezier_Float();\n";
		str += "$"+(obj.name)+".Attribute_Holder.reach.controller = Bezier_Float();\n";
		str += "$"+(obj.name)+".Attribute_Holder.shrug.controller = Bezier_Float();\n";
		)
	else
		(
		str = "";
		str += "$"+(obj.name)+".localworld.controller = Bezier_Float();\n";
		str += "$"+(obj.name)+".wristtwist.controller = Bezier_Float();\n";
		str += "$"+(obj.name)+".reach.controller = Bezier_Float();\n";
		str += "$"+(obj.name)+".shrug.controller = Bezier_Float();\n";
		)
	execute str;
	
)


-- -----------------------------------------------------------------------------------------------

/*
 * addFingerAttrs() - Adds finger attributes onto the object.
 *		doThumb is true if there are supposed to be thumb CA's
 *		numF	is the number of finger CA's to add
 *
 *    This defines a CA definiton.
 *	  Then it adds some custom data as an array of two sub arrays
 *			the first being an array of MAX's CAT_ParamBlock structs
 *			the second is an array of MAX's CAT_UIItem structs, 
 *				with one item per attribute.  This custom data stores info
 *				about the range, so other stuff, like my attrKey script can use 
 *				and get the range data back out.  
 *	  Finally the data is then actually added onto the object.  If the user has
 *		the EmptyModifer() aka Attribute Holder modifier installed, then it 
 *		will tack that onto the object.  Otherwise it adds it to the base object.
 */
fn addFingerAttrs obj doThumb numF =
(
    if (obj == undefined) then 
		(
		format ("charCreator INTERNAL ERROR: addFingerAttrs(), obj is undefined!\n");
		return();
		)

/*
--DON'T DO THIS SINCE WE'VE ALREADY DONE IT WHEN WE ADDED THE ARM ATTRS ON
		-- Make sure user installed the AttributeHolder modifier.
    if (EmptyModifier != undefined) then 
		(
			-- Add Attr Holder modifier
		addModifier obj (EmptyModifier());
		)
*/
		--
		-- main attr def stuff
		--
    str = "fingerAttrs = attributes Custom_Attributes\n"+
		"(\n"+
	    "parameters main rollout:fingerAttrsRO\n"+
		"	(\n";
		if (doThumb) then
			(
			str += "	thumb type:#float default:0.0 ui:thumbSpin\n"+
				   "	thumbtwist type:#float default:0.0 ui:thumbtwistSpin\n"+
				   "	thumbadj type:#float default:0.0 ui:thumbadjSpin\n";
			)
		if (numF > 1) then
			(
			str += "	spread type:#float default:0.0 ui:spreadSpin\n";
			)
		for i in 1 to numF do
			(
			str += "	finger"+(i as string)+" type:#float default:0.0 ui:finger"+(i as string)+"Spin\n";
			)
		str += "	)\n"+
		"   \n"+
		"rollout fingerAttrsRO \"Finger Anim Parameters\"\n"+ 
		"   (\n";
		if (doThumb) then
			(
			str +=	"   spinner 'thumbSpin' \"thumb\" type:#float range:[-100,100,0];\n"+
				    "   spinner 'thumbtwistSpin' \"thumbTwist\" type:#float range:[-100,100,0];\n"+
				    "   spinner 'thumbadjSpin' \"thumbAdj\" type:#float range:[-100,100,0];\n";
			)
		if (numF > 1) then
			(
			str +=	"   spinner 'spreadSpin' \"spread\" type:#float range:[-100,100,0];\n";
			)
		for i in 1 to numF do
			(
			str +=	"   spinner 'finger"+(i as string)+"Spin' \"finger"+(i as string)+"\" type:#float range:[-100,100,0];\n";
			)
	    str += "   )\n"+
		");\n"+
		"\n" ;
	execute str;

		--
		-- Add some custom def data like MAX does, so we can retrieve RANGE info later for things like my attrKey script.
		--
	str = "\n";
		if (doThumb) then
			(
			str += "pb1 = CAT_ParamBlock name:\"'thumb'\" type:\"Float\" ui:\"thumbSpin\" default:\"0.0\";\n"+
				   "pb2 = CAT_ParamBlock name:\"'thumbadj'\" type:\"Float\" ui:\"thumbadjSpin\" default:\"0.0\";\n"+
				   "pb2a = CAT_ParamBlock name:\"'thumbtwist'\" type:\"Float\" ui:\"thumbtwistSpin\" default:\"0.0\";\n";
			)		
		if (numF > 1) then
			(
			str += "pb3 = CAT_ParamBlock name:\"'spread'\" type:\"Float\" ui:\"spreadSpin\" default:\"0.0\";\n";
			)
		for i in 1 to numF do
			(
			str += "pb"+((i+3) as string)+" = CAT_ParamBlock name:\"'finger"+(i as string)+"'\" type:\"Float\" ui:\"finger"+(i as string)+"Spin\" default:\"0.0\";\n";
			)
		if (doThumb) then
			(
			str += "ui1 = CAT_UIItem ui:\"Spinner\" name:\"'thumb'\" type:\"Float\" range:\"[-100,100,0]\";\n"+
				   "ui2 = CAT_UIItem ui:\"Spinner\" name:\"'thumbadj'\" type:\"Float\" range:\"[-100,100,0]\";\n"+
				   "ui2a = CAT_UIItem ui:\"Spinner\" name:\"'thumbtwist'\" type:\"Float\" range:\"[-100,100,0]\";\n";
			)		
		if (numF > 1) then
			(
			str += "ui3 = CAT_UIItem ui:\"Spinner\" name:\"'spread'\" type:\"Float\" range:\"[-100,100,0]\";\n";
			)		
		for i in 1 to numF do
			(
			str += "ui"+((i+3) as string)+" = CAT_UIItem ui:\"Spinner\" name:\"'finger"+(i as string)+"'\" type:\"Float\" range:\"[-100,100,0]\";\n";
			)

		if (doThumb) then
			(
			str += "arr1 = #(pb1, pb2, pb2a";
--			if (numF <= 0) then
--				str += ");\n";
			)
		if (numF > 1) then
			(
			if (doThumb == false) then
				str += "arr1 = #(";
			else
				str += ",";
			str += "pb3";
			)
		if (numF == 1) then
			(
			if (doThumb == false) then
				str += "arr1 = #(";
			else
				str += ",";
			str += "pb4);\n";
			)
		else (
			for i in 1 to numF do
				(
				str += (", pb"+((i+3) as string));
				)
			str += ");\n";
			)
			
		if (doThumb) then
			(
			str += "arr2 = #(ui1, ui2, ui2a";
--			if (numF <= 0) then
--				str += ");\n";
			)
		if (numF > 1) then
			(
			if (doThumb == false) then
				str += "arr2 = #(";
			else
				str += ",";
			str += "ui3";
			)
		if (numF == 1) then
			(
			if (doThumb == false) then
				str += "arr1 = #(";
			else
				str += ",";
			str += "ui4);\n";
			)
		else (
			for i in 1 to numF do
				(
				str += (", ui"+((i+3) as string));
				)
			str += ");\n";
			)

		str += "theArr = #(arr1, arr2)\n"+
		"\n"+
		"custAttributes.setDefData fingerAttrs theArr;\n";
	execute str;
		
    if (EmptyModifier != undefined) then 
		str = "custAttributes.add $"+(obj.name)+".Attribute_Holder fingerAttrs;";
	else
		str = "custAttributes.add $"+(obj.name)+" fingerAttrs;";
	execute str;
		-- 
		-- Now add a default controller too
		--	
    if (EmptyModifier != undefined) then 
		(
		str = "";
		if (doThumb) then
			(
			str += "$"+(obj.name)+".Attribute_Holder.thumb.controller = Bezier_Float();\n";
			str += "$"+(obj.name)+".Attribute_Holder.thumbadj.controller = Bezier_Float();\n";
			str += "$"+(obj.name)+".Attribute_Holder.thumbtwist.controller = Bezier_Float();\n";
			)
		if (numF > 1) then
			str += "$"+(obj.name)+".Attribute_Holder.spread.controller = Bezier_Float();\n";
		for i in 1 to numF do
			str += "$"+(obj.name)+".Attribute_Holder.finger"+(i as string)+".controller = Bezier_Float();\n";
		)
	else
		(
		str = "";
		if (doThumb) then
			(
			str += "$"+(obj.name)+".thumb.controller = Bezier_Float();\n";
			str += "$"+(obj.name)+".thumbadj.controller = Bezier_Float();\n";
			str += "$"+(obj.name)+".thumbtwist.controller = Bezier_Float();\n";
			)
		if (numF > 1) then
			str += "$"+(obj.name)+".spread.controller = Bezier_Float();\n";
		for i in 1 to numF do
			str += "$"+(obj.name)+".finger"+(i as string)+".controller = Bezier_Float();\n";
		)
	execute str;

)



-- -----------------------------------------------------------------------------------------------

/*
 * addToeAttrs() - Adds toe attributes onto the object.
 *      doToe1  true if we have a toe 1 with special control
 *		numT	is the number of toe CA's to add
 *
 *    This defines a CA definiton.
 *	  Then it adds some custom data as an array of two sub arrays
 *			the first being an array of MAX's CAT_ParamBlock structs
 *			the second is an array of MAX's CAT_UIItem structs, 
 *				with one item per attribute.  This custom data stores info
 *				about the range, so other stuff, like my attrKey script can use 
 *				and get the range data back out.  
 *	  Finally the data is then actually added onto the object.  If the user has
 *		the EmptyModifer() aka Attribute Holder modifier installed, then it 
 *		will tack that onto the object.  Otherwise it adds it to the base object.
 */
fn addToeAttrs obj doToe1 numT =
(
    if (obj == undefined) then 
		(
		format ("charCreator INTERNAL ERROR: addToeAttrs(), obj is undefined!\n");
		return();
		)

/*
--DON'T DO THIS SINCE WE'VE ALREADY DONE IT WHEN WE ADDED THE ARM ATTRS ON
		-- Make sure user installed the AttributeHolder modifier.
    if (EmptyModifier != undefined) then 
		(
			-- Add Attr Holder modifier
		addModifier obj (EmptyModifier());
		)
*/
		--
		-- main attr def stuff
		--
    str = "toeAttrs = attributes ToeCustom_Attributes\n"+
		"(\n"+
	    "parameters main rollout:toeAttrsRO\n"+
		"	(\n";
		if (doToe1) then
			(
			str += "	toe1 type:#float default:0.0 ui:toe1Spin\n"+
				   "	toe1twist type:#float default:0.0 ui:toe1twistSpin\n"+
				   "	toe1adj type:#float default:0.0 ui:toe1adjSpin\n";
			)
		if (numT > 1) then
			(
			str += "	toespread type:#float default:0.0 ui:toespreadSpin\n";
			)
		for i in 2 to (numT+1) do
			(
			str += "	toe"+(i as string)+" type:#float default:0.0 ui:toe"+(i as string)+"Spin\n";
			)
		str += "	)\n"+
		"   \n"+
		"rollout toeAttrsRO \"Toe Anim Parameters\"\n"+ 
		"   (\n";
		if (doToe1) then
			(
			str +=	"   spinner 'toe1Spin' \"toe1\" type:#float range:[-100,100,0];\n"+
				    "   spinner 'toe1twistSpin' \"toe1Twist\" type:#float range:[-100,100,0];\n"+
				    "   spinner 'toe1adjSpin' \"toe1Adj\" type:#float range:[-100,100,0];\n";
			)
		if (numT > 1) then
			(
			str +=	"   spinner 'toespreadSpin' \"spread\" type:#float range:[-100,100,0];\n";
			)
		for i in 2 to (numT+1) do
			(
			str +=	"   spinner 'toe"+(i as string)+"Spin' \"toe"+(i as string)+"\" type:#float range:[-100,100,0];\n";
			)
	    str += "   )\n"+
		");\n"+
		"\n" ;
	execute str;


		--
		-- Add some custom def data like MAX does, so we can retrieve RANGE info later for things like my attrKey script.
		--
	str = "\n";
		if (doToe1) then
			(
			str += "pb1 = CAT_ParamBlock name:\"'toe1'\" type:\"Float\" ui:\"toe1Spin\" default:\"0.0\";\n"+
				   "pb2 = CAT_ParamBlock name:\"'toe1adj'\" type:\"Float\" ui:\"toe1adjSpin\" default:\"0.0\";\n"+
				   "pb2a = CAT_ParamBlock name:\"'toe1twist'\" type:\"Float\" ui:\"toe1twistSpin\" default:\"0.0\";\n";
			)		
		if (numT > 1) then
			(
			str += "pb3 = CAT_ParamBlock name:\"'toespread'\" type:\"Float\" ui:\"toespreadSpin\" default:\"0.0\";\n";
			)
		for i in 2 to (numT+1) do
			(
			str += "pb"+((i+3) as string)+" = CAT_ParamBlock name:\"'toe"+(i as string)+"'\" type:\"Float\" ui:\"toe"+(i as string)+"Spin\" default:\"0.0\";\n";
			)
		if (doToe1) then
			(
			str += "ui1 = CAT_UIItem ui:\"Spinner\" name:\"'toe1'\" type:\"Float\" range:\"[-100,100,0]\";\n"+
				   "ui2 = CAT_UIItem ui:\"Spinner\" name:\"'toe1adj'\" type:\"Float\" range:\"[-100,100,0]\";\n"+
				   "ui2a = CAT_UIItem ui:\"Spinner\" name:\"'toe1twist'\" type:\"Float\" range:\"[-100,100,0]\";\n";
			)		
		if (numT > 1) then
			(
			str += "ui3 = CAT_UIItem ui:\"Spinner\" name:\"'toespread'\" type:\"Float\" range:\"[-100,100,0]\";\n";
			)		
		for i in 2 to (numT+1) do
			(
			str += "ui"+((i+3) as string)+" = CAT_UIItem ui:\"Spinner\" name:\"'toe"+(i as string)+"'\" type:\"Float\" range:\"[-100,100,0]\";\n";
			)

		if (doToe1) then
			(
			str += "arr1 = #(pb1, pb2, pb2a";
			)
		if (numT > 1) then
			(
			if (doToe1 == false) then
				str += "arr1 = #(";
			else
				str += ",";
			str += "pb3";
			)
		if (numT == 1) then
			(
			if (doToe1 == false) then
				str += "arr1 = #(";
			else
				str += ",";
			str += "pb4);\n";
			)
		else (
			for i in 2 to (numT+1) do
				(
				str += (", pb"+((i+3) as string));
				)
			str += ");\n";
			)
			
		if (doToe1) then
			(
			str += "arr2 = #(ui1, ui2, ui2a";
			)
		if (numT > 1) then
			(
			if (doToe1 == false) then
				str += "arr2 = #(";
			else
				str += ",";
			str += "ui3";
			)
		if (numT == 1) then
			(
			if (doToe1 == false) then
				str += "arr1 = #(";
			else
				str += ",";
			str += "ui4);\n";
			)
		else (
			for i in 2 to (numT+1) do
				(
				str += (", ui"+((i+3) as string));
				)
			str += ");\n";
			)

		str += "theArr = #(arr1, arr2)\n"+
		"\n"+
		"custAttributes.setDefData toeAttrs theArr;\n";
	execute str;
		
    if (EmptyModifier != undefined) then 
		str = "custAttributes.add $"+(obj.name)+".Attribute_Holder toeAttrs;";
	else
		str = "custAttributes.add $"+(obj.name)+" toeAttrs;";
	execute str;
		-- 
		-- Now add a default controller too
		--	
    if (EmptyModifier != undefined) then 
		(
		str = "";
		if (doToe1) then
			(
			str += "$"+(obj.name)+".Attribute_Holder.toe1.controller = Bezier_Float();\n";
			str += "$"+(obj.name)+".Attribute_Holder.toe1adj.controller = Bezier_Float();\n";
			str += "$"+(obj.name)+".Attribute_Holder.toe1twist.controller = Bezier_Float();\n";
			)
		if (numT > 1) then
			str += "$"+(obj.name)+".Attribute_Holder.toespread.controller = Bezier_Float();\n";
		for i in 2 to (numT+1) do
			str += "$"+(obj.name)+".Attribute_Holder.toe"+(i as string)+".controller = Bezier_Float();\n";
		)
	else
		(
		str = "";
		if (doToe1) then
			(
			str += "$"+(obj.name)+".toe1.controller = Bezier_Float();\n";
			str += "$"+(obj.name)+".toe1adj.controller = Bezier_Float();\n";
			str += "$"+(obj.name)+".toe1twist.controller = Bezier_Float();\n";
			)
		if (numT > 1) then
			str += "$"+(obj.name)+".toespread.controller = Bezier_Float();\n";
		for i in 2 to (numT+1) do
			str += "$"+(obj.name)+".toe"+(i as string)+".controller = Bezier_Float();\n";
		)
	execute str;

)


-- -----------------------------------------------------------------------------------------------

/*
 * createHips() - Makes the hips object
 */
fn createHips =
(

    if (objExists(prefix+"_"+"hipCTRL")) then 
		(
		format "Hips already created.\n";
		execute("hipCTRL = $"+prefix+"_hipCTRL;");
		return 0;
		)
    if (objExists("$ptHips") != true) then 
		(
		format "WARNING: Can't create hips.  \"ptHips\" does not exist.\n";
		return 0;
		)

	-- Figure out Radius
	r = 0;
    if (objExists("$ptLLeg")) then 
		(
		r = $ptLLeg.pos.x - $ptHips.pos.x;
		r = r + (0.55*r);
		r = abs(r);
		) 
	else if (objExists("$ptRLeg")) then 
		(
		r = $ptRLeg.pos.x - $ptHips.pos.x;
		r = r + (0.55*r);
		r = abs(r);
		)
	if (r <= 0) then 
		(
		r = $ptHips.size*1.33;		
		)

    hipCTRL = Ngon radius:r cornerRadius:0 nsides:8 circular:off scribe:1 pos:$ptHips.pos isSelected:on;
	hipCTRL.name = (prefix+"_hipCTRL");
	hipCTRL.wirecolor = bodyColor;
	hipCTRL.position.controller = Position_XYZ();
	hipCTRL.rotation.controller = Euler_XYZ();
	convertToSplineShape hipCTRL;

    if (objExists(prefix+"_"+"placementCTRL")) then 
		(
		execute("hipCTRL.parent = $"+prefix+"_placementCTRL;");
		zeroOut hipCTRL;
		)
		
    format "Hips created.\n";
    return 1;
)

-- -----------------------------------------------------------------------------------------------

/*
 * createPlacement() - Makes the Ground and Placement objects
 */
fn createPlacement =
(
    FL = undefined;
	FR = undefined;
	
    if (objExists(prefix+"_"+"placementCTRL")) then 
		(
		format "Placement already exists.\n";
		return 0;
		)
    if (objExists(prefix+"_"+"groundCTRL")) then 
		(
		format "Placement already exists.\n";
		return 0;
		)

    if (objExists(prefix+"_"+"hipCTRL")) then 
		(
		execute("hipCTRL = $"+prefix+"_hipCTRL;");
		)
	
		-- Grab actual foot ctrls if they exist, pt locators otherwise.
	FL = execute("$"+prefix+"_footLCTRL");
	FR = execute("$"+prefix+"_footRCTRL");
	FL2 = execute("$"+prefix+"_footL2CTRL");
	FR2 = execute("$"+prefix+"_footR2CTRL");

	if ( hipCTRL == undefined or (FL == undefined and FR == undefined)) then 
		(
		format "WARNING: Can't create Placement.  Hips and Legs don't exist.\n";
		return 0;
		)

		-- Calculate center position of body at foot height, and radius.
	ctr = hipCTRL.pos;
	if (FL2 != undefined) then 
		(
		ctr.z = FL2.pos.z;
		r = FL2.pos.y - hipCTRL.pos.y;	-- for front legs use forward/back dist not left/right for radius
		)
	else if (FR2 != undefined) then 
		(
		ctr.z = FR2.pos.z;
		r = FR2.pos.y - hipCTRL.pos.y;
		)
	else if (FL != undefined) then 
		(
		ctr.z = FL.pos.z;
		r = FL.pos.x - hipCTRL.pos.x;
		)
	else
		(
		ctr.z = FR.pos.z;
		r = FR.pos.x - hipCTRL.pos.x;
		)
	-- The above figures center and a good approx of radius r for only 1 foot.
	-- but if we have 2 feet then we can figure r even better.
    if (FL != undefined and FR != undefined) then
		(
		r = length(FL.pos - FR.pos);
		r *= 1.5;
		)

    r = abs(r);
	r = r * 3.2;	-- make even bigger for placement

    placeCTRL = Ngon radius:r cornerRadius:0 nsides:12 circular:off scribe:1 pos:ctr isSelected:on;
	placeCTRL.name = (prefix+"_placementCTRL");
	placeCTRL.wirecolor = (bodyColor*0.2)+(baseColor*0.8);
	placeCTRL.position.controller = Position_XYZ();
	placeCTRL.rotation.controller = Euler_XYZ();
	convertToSplineShape placeCTRL;

    grndCTRL = Ngon radius:(r*0.65) cornerRadius:0 nsides:10 circular:off scribe:1 pos:ctr isSelected:on;
	grndCTRL.name = (prefix+"_groundCTRL");
	grndCTRL.wirecolor = (bodyColor*0.6)+(baseColor*0.4);
	grndCTRL.position.controller = Position_XYZ();
	grndCTRL.rotation.controller = Euler_XYZ();
	convertToSplineShape grndCTRL;

	-- Now parent things...
	
	hipCTRL.parent = placeCTRL;
	zeroOut hipCTRL;
	grndCTRL.parent = placeCTRL;
	zeroOut grndCTRL;
	if (FL != undefined) then
		(
		FL.parent = grndCTRL;
		zeroOut FL;
		)
	if (FR != undefined) then
		(
		FR.parent = grndCTRL;
		zeroOut FR;
		)
	if (FL2 != undefined) then
		(
		FL2.parent = grndCTRL;
		zeroOut FL2;
		)
	if (FR2 != undefined) then
		(
		FR2.parent = grndCTRL;
		zeroOut FR2;
		)
		

	format "Placement and ground created.\n";
	return 1;
)


-- -----------------------------------------------------------------------------------------------

/*
 * createLeg() - This creates the leg....
 */
fn createLeg LR =
(


    if (objExists("$pt"+LR+"Leg") != true) then 
		(
		format "WARNING: Can't create % Leg.  \"pt%Leg\" does not exist.\n" LR LR;
		return 0;
		)
    if (objExists("$pt"+LR+"Knee") != true) then 
		(
		format "WARNING: Can't create % Leg.  \"pt%Knee\" does not exist.\n" LR LR;
		return 0;
		)
    if (objExists("$pt"+LR+"Ankle") != true) then 
		(
		format "WARNING: Can't create % Leg.  \"pt%Ankle\" does not exist.\n" LR LR;
		return 0;
		)
    if (objExists("$pt"+LR+"Ball") != true) then 
		(
		format "WARNING: Can't create % Leg.  \"pt%Ball\" does not exist.\n" LR LR;
		return 0;
		)

    if (objExists("$pt"+LR+"Toe") != true) then 
		(
		format "WARNING: Can't create % Leg.  \"pt%Toe\" does not exist.\n" LR LR;
		return 0;
		)

   if (objExists(prefix+"_foot"+LR+"CTRL")) then 
		(
		format "Leg % already created.\n" LR;
		return 0;
		)


--
	if (LR == "L") then
		(
		ptLeg = $ptLLeg;
		ptKnee = $ptLKnee;
		ptAnkle = $ptLAnkle;
		ptBall = $ptLBall;
		ptToe = $ptLToe;		
		)
	else if (LR == "R") then 
		(
		ptLeg = $ptRLeg;
		ptKnee = $ptRKnee;
		ptAnkle = $ptRAnkle;
		ptBall = $ptRBall;
		ptToe = $ptRToe;		
		)
	else if (LR == "L2") then
		(
		ptLeg = $ptL2Leg;
		ptKnee = $ptL2Knee;
		ptAnkle = $ptL2Ankle;
		ptBall = $ptL2Ball;
		ptToe = $ptL2Toe;		
		)
	else (
		ptLeg = $ptR2Leg;
		ptKnee = $ptR2Knee;
		ptAnkle = $ptR2Ankle;
		ptBall = $ptR2Ball;
		ptToe = $ptR2Toe;		
		)


		-- Create the bones and internal stuff
	swvDist = 2.0 * length(ptAnkle.pos - ptToe.pos);
	ctrlRad = 1.2 * length(ptAnkle.pos - ptToe.pos);
	
	if (LR == "L" or LR == "L2") then
		sideColor = leftColor;
	else
		sideColor = rightColor;

    bUL = BoneSys.createBone ptLeg.pos ptKnee.pos [1,0,0]; 			-- upper leg
	bUL.name = (prefix+"_upperLeg"+LR+"BONE");
	boneSetup bUL true mixColor:sideColor;
    bLL = BoneSys.createBone ptKnee.pos ptAnkle.pos [1,0,0]; 			-- lower leg
	bLL.name = (prefix+"_lowerLeg"+LR+"BONE");
	boneSetup bLL true mixColor:sideColor;
    bFT = BoneSys.createBone ptAnkle.pos ptBall.pos [1,0,0]; 			-- foot
	bFT.name = (prefix+"_foot"+LR+"BONE");
	boneSetup bFT true mixColor:sideColor;
    bTO = BoneSys.createBone ptBall.pos ptToe.pos [1,0,0]; 				-- toe
	bTO.name = (prefix+"_toe"+LR+"BONE");
	boneSetup bTO true mixColor:sideColor;
	
	tipSize = bFT.width;
	pointSize = tipSize*4;
																		-- leg start zeroer
	pST = Point pos:ptLeg.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
	pST.name = (prefix+"_legStart"+LR+"INT");

    p2 = ptToe.pos;		-- Get a vector from ball to toe
	p1 = ptBall.pos;
    pn = p2 - p1;
	pn = normalize pn;  -- Normalize it

    bTP = BoneSys.createBone ptToe.pos (ptToe.pos + (pn*tipSize)) [1,0,0]; 	-- tip
	bTP.name = (prefix+"_toeTip"+LR+"BONE");
	boneSetup bTP false mixColor:sideColor;
	bTP.width = bTP.height = tipSize;	-- override
	
   
																			-- swivel ctrl
	pSWV = Point pos:(ptKnee.pos+[0,-swvDist,0]) isSelected:on centermarker:off axistripod:off cross:on Box:on constantscreensize:off drawontop:off size:pointSize;
	pSWV.name = (prefix+"_knee"+LR+"CTRL");
	
		-- Now link them
	bTP.parent = bTO;
	bTO.parent = bFT;
	bFT.parent = bLL;
	bLL.parent = bUL;
	bUL.parent = pST;
	if (hipCTRL != undefined) then
		pST.parent = hipCTRL;
	
		-- set some colors
	pST.wirecolor = internalColor;
	pSWV.wirecolor = sideColor;

		-- Create the IK chain solver/effector's.
	endEf1 = IKSys.ikChain bUL bFT "IKHiSolver";
	endEf1.name = (prefix+"_leg"+LR+"IKEndEffector");
	endEf1.transform.controller.SAParent = 0;
    endEf1.rotation = ptAnkle.rotation;
    endEf1.pos = ptAnkle.pos;
	
	endEf2 = IKSys.ikChain bFT bTO "IKHiSolver";
	endEf2.name = (prefix+"_foot"+LR+"IKEndEffector");
	endEf2.transform.controller.SAParent = 0;
    endEf2.rotation = ptAnkle.rotation;
    endEf2.pos = ptBall.pos;

	endEf3 = IKSys.ikChain bTO bTP "IKHiSolver";
	endEf3.name = (prefix+"_toe"+LR+"IKEndEffector");
	endEf3.transform.controller.SAParent = 0;
    endEf3.rotation = ptAnkle.rotation;
    endEf3.pos = ptToe.pos;
    endEf3.transform.controller.SAParent = 1;   -- set to start not IK goal Parent Space.

		-- Setup swivel control for knee
	endEf1.transform.controller.VHTarget = pSWV; 
		
		-- Make the foot CTRL curve
    footCTRL = Ngon radius:ctrlRad cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptAnkle.pos isSelected:on;
	footCTRL.name = (prefix+"_foot"+LR+"CTRL");
	footCTRL.wirecolor = sideColor;
--	footCTRL.scale.x = 0.60;	-- bad breaks even with xform ...sigh
		-- now reset the xform since we scaled.
	addModifier footCTRL (XForm());
	collapseStack footCTRL;
	footCTRL.position.controller = Position_XYZ();
	footCTRL.rotation.controller = Euler_XYZ();
	footCOPY = copy footCTRL;		-- make copy
	footCOPY.scale.x = 1.00;
	coordsys local (rotate footCOPY (eulerangles -90 0 90));	-- rotate
	footCOPY.scale.y = 0.50;
	addModifier footCOPY (XForm());
	collapseStack footCOPY;

	addAndWeld footCTRL footCOPY 0;		-- weld copy
	addModifier footCTRL (XForm());
	collapseStack footCTRL;

    pSWV.position.controller = Position_XYZ();
	pSWV.position = (ptKnee.pos+[0,-swvDist,0])
	pSWV.parent = footCTRL;
	zeroOut pSWV;

		-- Now set Rotation in case foot is at an angle in TOP view! !!!
    coordsys world footCTRL.rotation = ptAnkle.rotation
    if (LR == "L" or LR == "L2") then
        coordsys local rotate footCTRL (angleaxis -90 [1,0,0])
	else
        coordsys local rotate footCTRL (angleaxis 90 [1,0,0])
   	footCTRL.pos = ptAnkle.pos;
	


		-- Add CA's to foot
	addFootAttrs(footCTRL);

		-- now make our nice dummies for internal control
	pBL = Point pos:ptBall.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(pointSize*1.5);
	pBL.name = (prefix+"_ballRot"+LR+"INT");
	pBL.wirecolor = internalColor;		-- ball rotator
	pBL.rotation.controller = Euler_XYZ();
    pBL.rotation = footCTRL.rotation;
	pBL.position = ptBall.pos;

	pTO = Point pos:ptBall.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
	pTO.name = (prefix+"_toeRot"+LR+"INT");
	pTO.wirecolor = internalColor;		-- toe rotator
	pTO.rotation.controller = Euler_XYZ();
    pTO.rotation = footCTRL.rotation;
	pTO.position = ptBall.pos;


	pHE = Point pos:ptAnkle.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
	pHE.name = (prefix+"_heelRot"+LR+"INT");
	pHE.wirecolor = internalColor;		-- heel master rotator
	pHE.rotation.controller = Euler_XYZ();
    pHE.rotation = footCTRL.rotation;
	pHE.position = ptAnkle.pos;
	pHE.position.z = ptBall.pos.z;	-- this should be on "ground" level even if foot pivot is raised etc..


	pTP = Point pos:ptToe.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
	pTP.name = (prefix+"_tipRot"+LR+"INT");
	pTP.wirecolor = internalColor;		-- toe tip rotator
	pTP.rotation.controller = Euler_XYZ();
    pTP.rotation = footCTRL.rotation;
	pTP.position = ptToe.pos;

	
		-- link the dummies and the endEffectors
	endEf1.parent = pBL;
	endEf2.parent = pBL;
	endEf3.parent = pTO;
	pBL.parent = pTP;
	pTO.parent = pTP;
	pTP.parent = pHE;
	pHE.parent = footCTRL;
		-- If the ground ctrl exists, parent foot to it...		
	if (objExists(prefix+"_groundCTRL")) then
		(
		execute("$"+prefix+"_foot"+LR+"CTRL.parent = $"+prefix+"_groundCTRL;");
		zeroOut footCTRL;
		)
		
		-- woohoo!  All we have to do at this point is do a nice reactor stuff on the dummies
    if (EmptyModifier != undefined) then 
		(
		rollCnt = footCTRL.Attribute_Holder.roll.controller;
		toeRotateCnt = footCTRL.Attribute_Holder.toeRotate.controller;
		)
	else
		(
		rollCnt = footCTRL.roll.controller;
		toeRotateCnt = footCTRL.toeRotate.controller;
		)

		--
		-- Toe rotator reaction
		--
	cnt = pTO.rotation.X_Rotation.controller = Float_Reactor();
	reactTo cnt toeRotateCnt;
	cnt.reactions.setName 0 "zero";
	cnt.reactions.setValueAsFloat 0 0;	-- when toeRotate is 0...
	cnt.reactions.setFloatState 0 0;	--    xrot is 0.

	toeRotateCnt.value = 180;
	cnt.reactions.create name:"up";
	cnt.reactions.setFloatState 0 (-1.5);	--    xrot is pi/2.
	
	toeRotateCnt.value = -180;
	cnt.reactions.create name:"down";
	cnt.reactions.setFloatState 0 (1.5);	--    xrot is pi/2.

	toeRotateCnt.value = 0;

		--
		-- Heel rotator reaction
		--
	cnt = pHE.rotation.X_Rotation.controller = Float_Reactor();
	reactTo cnt rollCnt;
	cnt.reactions.setName 0 "zero";
	cnt.reactions.setValueAsFloat 0 0;	-- when roll is 0...
	cnt.reactions.setFloatState 0 0;	--    xrot is 0.

	rollCnt.value = -100;
	cnt.reactions.create name:"up";
	cnt.reactions.setFloatState 0 (-1.15);	--    xrot is about -65 deg

	rollCnt.value = 0;

		--
		-- Ball rotator reaction
		--
	cnt = pBL.rotation.X_Rotation.controller = Float_Reactor();
	reactTo cnt rollCnt;
	cnt.reactions.setName 0 "zero";
	cnt.reactions.setValueAsFloat 0 0;	-- when roll is 0...
	cnt.reactions.setFloatState 0 0;	--    xrot is 0.

	rollCnt.value = 50;
	cnt.reactions.create name:"up";
	cnt.reactions.setFloatState 0 (1.12);	--    xrot is about 65 deg

	rollCnt.value = 0;

		--
		-- ToeTip rotator reaction
		--
	cnt = pTP.rotation.X_Rotation.controller = Float_Reactor();
	reactTo cnt rollCnt;
	cnt.reactions.setName 0 "zero";
	cnt.reactions.setValueAsFloat 0 0;	-- when roll is 0...
	cnt.reactions.setFloatState 0 0;	--    xrot is 0.

	rollCnt.value = 50;
	cnt.reactions.create name:"hold";
	cnt.reactions.setFloatState 0 (0.0);	--    xrot is still 0

	rollCnt.value = 100;
	cnt.reactions.create name:"up";
	cnt.reactions.setFloatState 0 (1.25);	--    xrot is about 70 deg

	rollCnt.value = 0;

		-- Finally if the stretchyLegs options is on do them!
	if (objExists(prefix+"_"+"hipCTRL")) then
		master = (prefix+"_"+"hipCTRL");
	else
		master = pST.name;

	if (stretchyLegs == true) then 
		makeStretchyBones bUL bLL bFT endEf1 footCTRL ("$"+footCTRL.name+" $"+master+" ");

    format "% Leg created.\n" LR;
    return 1;
)


-- -----------------------------------------------------------------------------------------------

/*
 * createSpine() - Makes spine ctrls
 */
fn createSpine =
(
    if (objExists(prefix+"_"+"hipCTRL") != true) then 
		(
		format "WARNING: Can't create Spine Controls, Hips don't exist.\n";
		return 0;
		)

		-- make sure global variable is up to date
    hipCTRL = execute("$"+prefix+"_"+"hipCTRL");
	lastSpine = undefined;

		-- Figure radius of controls
	if ($ptLLeg != undefined) then
		r = $ptLLeg.pos.x - hipCTRL.pos.x;
	else if ($ptRLeg != undefined) then
		r = $ptRLeg.pos.x - hipCTRL.pos.x;
	else
		r = $ptLLeg.size*2;
	r *= 1.25;		

	if ($ptS1 != undefined) then
		(
	    bS0 = BoneSys.createBone hipCTRL.pos $ptS1.pos [1,0,0];
		bS0.name = (prefix+"_hipBONE");
		bS0.rotation.controller = Euler_XYZ();
		boneSetup bS0 true mixColor:bodyColor;
		bS0.parent = hipCTRL;
				
		if ($ptS2 != undefined) then
			(
			cS1 = Ngon radius:r cornerRadius:0 nsides:6 circular:off scribe:1 pos:$ptS1.pos isSelected:on;
			cS1.name = (prefix+"_spine01CTRL");
			cS1.wirecolor = (bodyColor*0.8)+(bodyColor2*0.2);
			cS1.position.controller = Position_XYZ();
			cS1.rotation.controller = Euler_XYZ();
			convertToSplineShape cS1;

		    bS1 = BoneSys.createBone $ptS1.pos $ptS2.pos [1,0,0];
			bS1.name = (prefix+"_spine01BONE");
			bS1.rotation.controller = Euler_XYZ();
			boneSetup bS1 true mixColor:((bodyColor*0.8)+(bodyColor2*0.2));


			pF = Point pos:$ptS1.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.25*r);
			pF.name = (prefix+"_spine1FixINT");
			pF.wirecolor = internalColor;
			cS1.rotation = bS1.rotation;
			coordsys local (rotate cS1 (eulerangles 0 90 0) );
			cS1.position = $ptS1.pos;
			pF.parent = cS1;
			bS1.parent = pF;
			cS1.parent = hipCTRL;

			zeroOut cS1;
			lastSpine = cS1;

			if ($ptS3 != undefined) then
				(
				cS2 = Ngon radius:r cornerRadius:0 nsides:6 circular:off scribe:1 pos:$ptS2.pos isSelected:on;
				cS2.name = (prefix+"_spine02CTRL");
				cS2.wirecolor = (bodyColor*0.5)+(bodyColor2*0.5);
				cS2.position.controller = Position_XYZ();
				cS2.rotation.controller = Euler_XYZ();
				convertToSplineShape cS2;
	
			    bS2 = BoneSys.createBone $ptS2.pos $ptS3.pos [1,0,0];
				bS2.name = (prefix+"_spine02BONE");
				bS2.rotation.controller = Euler_XYZ();
				boneSetup bS2 true mixColor:((bodyColor*0.5)+(bodyColor2*0.5));

				pF = Point pos:$ptS2.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.25*r);
				pF.name = (prefix+"_spine2FixINT");
				pF.wirecolor = internalColor;
				cS2.rotation = bS2.rotation;
				coordsys local (rotate cS2 (eulerangles 0 90 0) );
				cS2.position = $ptS2.pos;
				pF.parent = cS2;
				bS2.parent = pF;
				cS2.parent = cS1;
	
				zeroOut cS2;
				lastSpine = cS2;


				if ($ptS4 != undefined) then
					(
					cS3 = Ngon radius:r cornerRadius:0 nsides:6 circular:off scribe:1 pos:$ptS3.pos isSelected:on;
					cS3.name = (prefix+"_spine03CTRL");
					cS3.wirecolor = (bodyColor*0.2)+(bodyColor2*0.8);
					cS3.position.controller = Position_XYZ();
					cS3.rotation.controller = Euler_XYZ();
					convertToSplineShape cS3;
		
				    bS3 = BoneSys.createBone $ptS3.pos $ptS4.pos [1,0,0];
					bS3.name = (prefix+"_spine03BONE");
					bS3.rotation.controller = Euler_XYZ();
					boneSetup bS3 true mixColor:((bodyColor*0.2)+(bodyColor2*0.8));

					pF = Point pos:$ptS3.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.25*r);
					pF.name = (prefix+"_spine3FixINT");
					pF.wirecolor = internalColor;
					cS3.rotation = bS3.rotation;
					coordsys local (rotate cS3 (eulerangles 0 90 0) );
					cS3.position = $ptS3.pos;
					pF.parent = cS3;
					bS3.parent = pF;
					cS3.parent = cS2;

					zeroOut cS3;
					lastSpine = cS3;

					if ($ptS5 != undefined) then
						(
						cS4 = Ngon radius:r cornerRadius:0 nsides:6 circular:off scribe:1 pos:$ptS4.pos isSelected:on;
						cS4.name = (prefix+"_spine04CTRL");
						cS4.wirecolor = (bodyColor*0.15)+(bodyColor2*0.85);
						cS4.position.controller = Position_XYZ();
						cS4.rotation.controller = Euler_XYZ();
						convertToSplineShape cS4;
			
					    bS4 = BoneSys.createBone $ptS4.pos $ptS5.pos [1,0,0];
						bS4.name = (prefix+"_spine04BONE");
						bS4.rotation.controller = Euler_XYZ();
						boneSetup bS4 true mixColor:((bodyColor*0.15)+(bodyColor2*0.85));

						pF = Point pos:$ptS4.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.25*r);
						pF.name = (prefix+"_spine4FixINT");
						pF.wirecolor = internalColor;
						cS4.rotation = bS4.rotation;
						coordsys local (rotate cS4 (eulerangles 0 90 0) );
						cS4.position = $ptS4.pos;
						pF.parent = cS4;
						bS4.parent = pF;
						cS4.parent = cS3;

						zeroOut cS4;
						lastSpine = cS4;

						if ($ptS6 != undefined) then
							(
							cS5 = Ngon radius:r cornerRadius:0 nsides:6 circular:off scribe:1 pos:$ptS5.pos isSelected:on;
							cS5.name = (prefix+"_spine05CTRL");
							cS5.wirecolor = (bodyColor*0.1)+(bodyColor2*0.9);
							cS5.position.controller = Position_XYZ();
							cS5.rotation.controller = Euler_XYZ();
							convertToSplineShape cS5;
				
						    bS5 = BoneSys.createBone $ptS5.pos $ptS6.pos [1,0,0];
							bS5.name = (prefix+"_spine05BONE");
							bS5.rotation.controller = Euler_XYZ();
							boneSetup bS5 true mixColor:((bodyColor*0.1)+(bodyColor2*0.9));

							pF = Point pos:$ptS5.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.25*r);
							pF.name = (prefix+"_spine5FixINT");
							pF.wirecolor = internalColor;
							cS5.rotation = bS5.rotation;
							coordsys local (rotate cS5 (eulerangles 0 90 0) );
							cS5.position = $ptS5.pos;
							pF.parent = cS5;
							bS5.parent = pF;
							cS5.parent = cS4;

							zeroOut cS5;
							lastSpine = cS5;

							if ($ptS7 != undefined) then
								(
								cS6 = Ngon radius:r cornerRadius:0 nsides:6 circular:off scribe:1 pos:$ptS6.pos isSelected:on;
								cS6.name = (prefix+"_spine05CTRL");
								cS6.wirecolor = (bodyColor*0.05)+(bodyColor2*0.95);
								cS6.position.controller = Position_XYZ();
								cS6.rotation.controller = Euler_XYZ();
								convertToSplineShape cS6;
					
							    bS6 = BoneSys.createBone $ptS6.pos $ptS7.pos [1,0,0];
								bS6.name = (prefix+"_spine06BONE");
								bS6.rotation.controller = Euler_XYZ();
								boneSetup bS6 true mixColor:((bodyColor*0.05)+(bodyColor2*0.95));

								pF = Point pos:$ptS6.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.25*r);
								pF.name = (prefix+"_spine6FixINT");
								pF.wirecolor = internalColor;
								cS6.rotation = bS6.rotation;
								coordsys local (rotate cS6 (eulerangles 0 90 0) );
								cS6.position = $ptS6.pos;
								pF.parent = cS6;
								bS6.parent = pF;
								cS6.parent = cS5;

								zeroOut cS6;
								lastSpine = cS6;
								)

							)

						)

					)

				)

			)

		)

	-- Now if we made front legs...parent them to the last spine bone control...
	--
	pSTL2 = execute("$"+prefix+"_legStartL2INT");
	pSTR2 = execute("$"+prefix+"_legStartR2INT");
	if (lastSpine != undefined and pSTL2 != undefined) then
		pSTL2.parent = lastSpine;
	if (lastSpine != undefined and pSTR2 != undefined) then
		pSTR2.parent = lastSpine;

	return 1;
)


-- -----------------------------------------------------------------------------------------------


/*
 * createHead() - Makes the neck and head ctrls
 */
fn createHead =
(
    if (objExists(prefix+"_headCTRL")) then 
		(
		format "WARNING: Head and Neck already created.\n";
		return 0;
		)
    
    if (objExists("$ptHead") != true) then 
		(
		format "WARNING: Can't create Head and Neck Controls, ptHead doesn't exist.\n";
		return 0;
		)
    if (objExists("$ptHeadTop") != true) then 
		(
		format "WARNING: Can't create Head and Neck Controls, ptHeadTop doesn't exist.\n";
		return 0;
		)

		-- figure out where the internal dummy starts at
	if ($ptNeck1 != undefined) then   -- either there is a 1st neck or no necks
		posST = $ptNeck1.pos;
	else
		posST = $ptHead.pos;

	cNeck1 = cNeck2 = undefined;

	tipSize = 0.17 * (length($ptHeadTop.pos - $ptHead.pos));
	pointSize = tipSize*4;

		-- Make the starting internal neck/head dummys
	pST = Point pos:posST isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
	pST.name = (prefix+"_headStartINT");
	pST.wirecolor = internalColor;

	pPC = Point pos:posST isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
	pPC.name = (prefix+"_headConstraintINT");
	pPC.wirecolor = internalColor;

		-- figure out what will be parent
	if (objExists("$"+prefix+"_spine07BONE")) then 
		parST = execute("$"+prefix+"_spine07BONE");
	else if (objExists("$"+prefix+"_spine06BONE")) then 
		parST = execute("$"+prefix+"_spine06BONE");
	else if (objExists("$"+prefix+"_spine05BONE")) then 
		parST = execute("$"+prefix+"_spine05BONE");
	else if (objExists("$"+prefix+"_spine04BONE")) then 
		parST = execute("$"+prefix+"_spine04BONE");
	else if (objExists("$"+prefix+"_spine03BONE")) then 
		parST = execute("$"+prefix+"_spine03BONE");
	else if (objExists("$"+prefix+"_spine02BONE")) then 
		parST = execute("$"+prefix+"_spine02BONE");
	else if (objExists("$"+prefix+"_spine01BONE")) then 
		parST = execute("$"+prefix+"_spine01BONE");
	else if (objExists("$"+prefix+"_hipBONE")) then 
		parST = execute("$"+prefix+"_hipBONE");
	else if (objExists("$"+prefix+"_hipCTRL")) then 
		parST = execute("$"+prefix+"_hipCTRL");
	else
		parST = undefined;

		-- set parent
	if (parST != undefined) then
		pST.parent = parST;

		-- Now constrain the pPC head node to the pST node which is parented.
		-- For independent heads, we truly do a POS constraint 
		--	otherwise, we'll just parent it...
	if (headIndep) then
		(
		pCntl = pPC.pos.controller = position_constraint();
		pCI = pCntl.constraints;
		pCI.appendTarget pST 50;
		)
	else (
		pPC.parent = pST;
		)

		-- Figure radius for controls
	r = $ptHeadTop.pos.z - $ptHead.pos.z;
	

		-- At this point everything gets built onto the pPC node...
		-- So we just make our neck and head bones and ctrls now...
	cHead = Ngon radius:(r*1.2) cornerRadius:0 nsides:8 circular:off scribe:1 pos:$ptHead.pos isSelected:on;
	cHead.name = (prefix+"_headCTRL");
	cHead.wirecolor = headColor;
	cHead.position.controller = Position_XYZ();
	cHead.rotation.controller = Euler_XYZ();
	convertToSplineShape cHead;

    bHead = BoneSys.createBone $ptHead.pos $ptHeadTop.pos [1,0,0];
	bHead.name = (prefix+"_headBONE");
	bHead.rotation.controller = Euler_XYZ();
	boneSetup bHead true mixColor:headColor;
		-- Override head fin size for a little bit larger for head...
	bHead.sidefinssize = r*0.6;
	bHead.frontfinsize = r*0.6;
	bHead.backfinsize = r*0.6;

    bHeadTip = BoneSys.createBone $ptHeadTop.pos ($ptHeadTop.pos+[0,0,tipSize]) [1,0,0];
	bHeadTip.name = (prefix+"_headTipBONE");
	boneSetup bHeadTip false mixColor:headColor;
		-- override stuff for tip bone...
	bHeadTip.width = bHeadTip.height = tipSize;	-- override

	cHead.rotation = bHead.rotation;
	coordsys local (rotate cHead (eulerangles 0 90 0) );
	cHead.position = $ptHead.pos;
	bHead.parent = cHead;
	bHeadTip.parent = bHead;

    lastNeck = pPC;     


	if ($ptNeck1 != undefined) then
		(
		cNeck1 = Ngon radius:r cornerRadius:0 nsides:8 circular:off scribe:1 pos:$ptNeck1.pos isSelected:on;
		cNeck1.name = (prefix+"_neck1CTRL");
		cNeck1.wirecolor = (headColor*0.2)+(bodyColor2*0.8);
		cNeck1.position.controller = Position_XYZ();
		cNeck1.rotation.controller = Euler_XYZ();
		convertToSplineShape cNeck1;
	
		if ($ptNeck2 != undefined) then
			p2 = $ptNeck2.pos;
		else
			p2 = $ptHead.pos;


		pF = Point pos:$ptNeck1.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
		pF.name = (prefix+"_neck1FixINT");
		pF.wirecolor = internalColor;
			
	    bNeck1 = BoneSys.createBone $ptNeck1.pos p2 [1,0,0];
		bNeck1.name = (prefix+"_neck1BONE");
		bNeck1.rotation.controller = Euler_XYZ();
		boneSetup bNeck1 true mixColor:((headColor*0.2)+(bodyColor2*0.8));

		cNeck1.rotation = bNeck1.rotation;
		coordsys local (rotate cNeck1 (eulerangles 0 90 0) );
		cNeck1.position = $ptNeck1.pos;
		pF.parent = cNeck1;
		bNeck1.parent = pF;
		cNeck1.parent = pPC;
		zeroOut cNeck1;
	    lastNeck = cNeck1;     
	
		
		if ($ptNeck2 != undefined) then
			(
			cNeck2 = Ngon radius:r cornerRadius:0 nsides:8 circular:off scribe:1 pos:$ptNeck2.pos isSelected:on;
			cNeck2.name = (prefix+"_neck2CTRL");
			cNeck2.wirecolor = (headColor*0.3)+(bodyColor2*0.7);
			cNeck2.position.controller = Position_XYZ();
			cNeck2.rotation.controller = Euler_XYZ();
			convertToSplineShape cNeck2;
		
			if ($ptNeck3 != undefined) then
				p2 = $ptNeck3.pos;
			else
				p2 = $ptHead.pos;

			pF = Point pos:$ptNeck2.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
			pF.name = (prefix+"_neck2FixINT");
			pF.wirecolor = internalColor;
	
		    bNeck2 = BoneSys.createBone $ptNeck2.pos p2 [1,0,0];
			bNeck2.name = (prefix+"_neck2BONE");
			bNeck2.rotation.controller = Euler_XYZ();
			boneSetup bNeck2 true mixColor:((headColor*0.3)+(bodyColor2*0.7));

			cNeck2.rotation = bNeck2.rotation;
			coordsys local (rotate cNeck2 (eulerangles 0 90 0) );
			cNeck2.position = $ptNeck2.pos;
			pF.parent = cNeck2;
			bNeck2.parent = pF;
			cNeck2.parent = cNeck1;
			zeroOut cNeck2;
		    lastNeck = cNeck2;

			if ($ptNeck3 != undefined) then
				(
				cNeck3 = Ngon radius:r cornerRadius:0 nsides:8 circular:off scribe:1 pos:$ptNeck3.pos isSelected:on;
				cNeck3.name = (prefix+"_neck3CTRL");
				cNeck3.wirecolor = (headColor*0.4)+(bodyColor2*0.6);
				cNeck3.position.controller = Position_XYZ();
				cNeck3.rotation.controller = Euler_XYZ();
				convertToSplineShape cNeck3;
			
				if ($ptNeck4 != undefined) then
					p2 = $ptNeck4.pos;
				else
					p2 = $ptHead.pos;
		
				pF = Point pos:$ptNeck3.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
				pF.name = (prefix+"_neck3FixINT");
				pF.wirecolor = internalColor;

			    bNeck3 = BoneSys.createBone $ptNeck3.pos p2 [1,0,0];
				bNeck3.name = (prefix+"_neck3BONE");
				bNeck3.rotation.controller = Euler_XYZ();
				boneSetup bNeck3 true mixColor:((headColor*0.4)+(bodyColor2*0.6));

				cNeck3.rotation = bNeck3.rotation;
				coordsys local (rotate cNeck3 (eulerangles 0 90 0) );
				cNeck3.position = $ptNeck3.pos;
				pF.parent = cNeck3;
				bNeck3.parent = pF;
				cNeck3.parent = cNeck2;
				zeroOut cNeck3;
			    lastNeck = cNeck3;

				if ($ptNeck4 != undefined) then
					(
					cNeck4 = Ngon radius:r cornerRadius:0 nsides:8 circular:off scribe:1 pos:$ptNeck4.pos isSelected:on;
					cNeck4.name = (prefix+"_neck4CTRL");
					cNeck4.wirecolor = (headColor*0.5)+(bodyColor2*0.5);
					cNeck4.position.controller = Position_XYZ();
					cNeck4.rotation.controller = Euler_XYZ();
					convertToSplineShape cNeck4;
				
					if ($ptNeck5 != undefined) then
						p2 = $ptNeck5.pos;
					else
						p2 = $ptHead.pos;

					pF = Point pos:$ptNeck4.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
					pF.name = (prefix+"_neck4FixINT");
					pF.wirecolor = internalColor;
			
				    bNeck4 = BoneSys.createBone $ptNeck4.pos p2 [1,0,0];
					bNeck4.name = (prefix+"_neck4BONE");
					bNeck4.rotation.controller = Euler_XYZ();
					boneSetup bNeck4 true mixColor:((headColor*0.5)+(bodyColor2*0.5));

					cNeck4.rotation = bNeck4.rotation;
					coordsys local (rotate cNeck4 (eulerangles 0 90 0) );
					cNeck4.position = $ptNeck4.pos;
					pF.parent = cNeck4;
					bNeck4.parent = pF;
					cNeck4.parent = cNeck3;
					zeroOut cNeck4;
				    lastNeck = cNeck4;
	
					if ($ptNeck5 != undefined) then
						(
						cNeck5 = Ngon radius:r cornerRadius:0 nsides:8 circular:off scribe:1 pos:$ptNeck5.pos isSelected:on;
						cNeck5.name = (prefix+"_neck5CTRL");
						cNeck5.wirecolor = (headColor*0.6)+(bodyColor2*0.4);
						cNeck5.position.controller = Position_XYZ();
						cNeck5.rotation.controller = Euler_XYZ();
						convertToSplineShape cNeck5;
					
						if ($ptNeck6 != undefined) then
							p2 = $ptNeck6.pos;
						else
							p2 = $ptHead.pos;
				
						pF = Point pos:$ptNeck5.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
						pF.name = (prefix+"_neck5FixINT");
						pF.wirecolor = internalColor;
	
					    bNeck5 = BoneSys.createBone $ptNeck5.pos p2 [1,0,0];
						bNeck5.name = (prefix+"_neck5BONE");
						bNeck5.rotation.controller = Euler_XYZ();
						boneSetup bNeck5 true mixColor:((headColor*0.6)+(bodyColor2*0.5));

						cNeck5.rotation = bNeck5.rotation;
						coordsys local (rotate cNeck5 (eulerangles 0 90 0) );
						cNeck5.position = $ptNeck5.pos;
						pF.parent = cNeck5;
						bNeck5.parent = pF;
						cNeck5.parent = cNeck4;
						zeroOut cNeck5;
					    lastNeck = cNeck5;
		
						if ($ptNeck6 != undefined) then
							(
							cNeck6 = Ngon radius:r cornerRadius:0 nsides:8 circular:off scribe:1 pos:$ptNeck6.pos isSelected:on;
							cNeck6.name = (prefix+"_neck6CTRL");
							cNeck6.wirecolor = (headColor*0.7)+(bodyColor2*0.3);
							cNeck6.position.controller = Position_XYZ();
							cNeck6.rotation.controller = Euler_XYZ();
							convertToSplineShape cNeck6;
						
							if ($ptNeck7 != undefined) then
								p2 = $ptNeck7.pos;
							else
								p2 = $ptHead.pos;
					
							pF = Point pos:$ptNeck6.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
							pF.name = (prefix+"_neck6FixINT");
							pF.wirecolor = internalColor;

						    bNeck6 = BoneSys.createBone $ptNeck6.pos p2 [1,0,0];
							bNeck6.name = (prefix+"_neck6BONE");
							bNeck6.rotation.controller = Euler_XYZ();
							boneSetup bNeck6 true mixColor:((headColor*0.7)+(bodyColor2*0.3));

							cNeck6.rotation = bNeck6.rotation;
							coordsys local (rotate cNeck6 (eulerangles 0 90 0) );
							cNeck6.position = $ptNeck6.pos;
							pF.parent = cNeck6;
							bNeck6.parent = pF;
							cNeck6.parent = cNeck5;
							zeroOut cNeck6;
						    lastNeck = cNeck6;
			
							if ($ptNeck7 != undefined) then
								(
								cNeck7 = Ngon radius:r cornerRadius:0 nsides:8 circular:off scribe:1 pos:$ptNeck7.pos isSelected:on;
								cNeck7.name = (prefix+"_neck7CTRL");
								cNeck7.wirecolor = (headColor*0.8)+(bodyColor2*0.2);
								cNeck7.position.controller = Position_XYZ();
								cNeck7.rotation.controller = Euler_XYZ();
								convertToSplineShape cNeck7;
							
								if ($ptNeck8 != undefined) then
									p2 = $ptNeck8.pos;
								else
									p2 = $ptHead.pos;
						
								pF = Point pos:$ptNeck7.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
								pF.name = (prefix+"_neck7FixINT");
								pF.wirecolor = internalColor;
	
							    bNeck7 = BoneSys.createBone $ptNeck7.pos p2 [1,0,0];
								bNeck7.name = (prefix+"_neck7BONE");
								bNeck7.rotation.controller = Euler_XYZ();
								boneSetup bNeck7 true mixColor:((headColor*0.8)+(bodyColor2*0.2));

								cNeck7.rotation = bNeck7.rotation;
								coordsys local (rotate cNeck7 (eulerangles 0 90 0) );
								cNeck7.position = $ptNeck7.pos;
								pF.parent = cNeck7;
								bNeck7.parent = pF;
								cNeck7.parent = cNeck6;
								zeroOut cNeck7;
							    lastNeck = cNeck7;
				
								if ($ptNeck8 != undefined) then
									(
									cNeck8 = Ngon radius:r cornerRadius:0 nsides:8 circular:off scribe:1 pos:$ptNeck8.pos isSelected:on;
									cNeck8.name = (prefix+"_neck8CTRL");
									cNeck8.wirecolor = (headColor*0.9)+(bodyColor2*0.1);
									cNeck8.position.controller = Position_XYZ();
									cNeck8.rotation.controller = Euler_XYZ();
									convertToSplineShape cNeck8;
								
									p2 = $ptHead.pos;
							
									pF = Point pos:$ptNeck8.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
									pF.name = (prefix+"_neck8FixINT");
									pF.wirecolor = internalColor;
		
								    bNeck8 = BoneSys.createBone $ptNeck8.pos p2 [1,0,0];
									bNeck8.name = (prefix+"_neck7BONE");
									bNeck8.rotation.controller = Euler_XYZ();
									boneSetup bNeck8 true mixColor:((headColor*0.9)+(bodyColor2*0.1));

									cNeck8.rotation = bNeck8.rotation;
									coordsys local (rotate cNeck8 (eulerangles 0 90 0) );
									cNeck8.position = $ptNeck8.pos;
									pF.parent = cNeck8;
									bNeck8.parent = pF;
									cNeck8.parent = cNeck7;
									zeroOut cNeck8;
								    lastNeck = cNeck8;
					
									)

								)
	
							)

						)

					)
	
				)

			)
		
		)

    cHead.parent = lastNeck;
	zeroOut cHead;
	
	format "Head and Neck created.\n";	
    return 1;
)


-- -----------------------------------------------------------------------------------------------


/*
 * createArm() - Makes a R or L arm setup
 */
fn createArm LR =
(
    if (objExists("$pt"+LR+"Clav") != true) then 
		(
		format "WARNING: Can't create % Arm.  \"pt%Clav\" does not exist.\n" LR LR;
		return 0;
		)
    if (objExists("$pt"+LR+"UpArm") != true) then 
		(
		format "WARNING: Can't create % Arm.  \"pt%UpArm\" does not exist.\n" LR LR;
		return 0;
		)
    if (objExists("$pt"+LR+"LoArm") != true) then 
		(
		format "WARNING: Can't create % Arm.  \"pt%LoArm\" does not exist.\n" LR LR;
		return 0;
		)
    if (objExists("$pt"+LR+"Hand") != true) then 
		(
		format "WARNING: Can't create % Arm.  \"pt%Hand\" does not exist.\n" LR LR;
		return 0;
		)
    if (objExists("$pt"+LR+"HandTip") != true) then 
		(
		format "WARNING: Can't create % Arm.  \"pt%HandTip\" does not exist.\n" LR LR;
		return 0;
		)

   if (objExists(prefix+"_hand"+LR+"CTRL")) then 
		(
		format "% Arm already created.\n" LR;
		return 0;
		)

		-- Setup variables for the proper side
	if (LR == "L") then
		(
		ptClav = $ptLClav;
		ptUpArm = $ptLUpArm;
		ptLoArm = $ptLLoArm;
		ptHand = $ptLHand;
		ptHandTip = $ptLHandTip;
		neg = 1;
		sideColor = leftColor;
		)
	else (
		ptClav = $ptRClav;
		ptUpArm = $ptRUpArm;
		ptLoArm = $ptRLoArm;
		ptHand = $ptRHand;
		ptHandTip = $ptRHandTip;
		neg = -1;
		sideColor = rightColor;
		)

		-- Figure out sizes and pos for stuff
	swvDist = 2.0 * length(ptUpArm.pos - ptLoArm.pos);
	ctrlRad = 1.2 * length(ptHand.pos - ptHandTip.pos);

	tipSize = 0.17 * (length(ptHand.pos - ptLoArm.pos));
	pointSize = tipSize*4;


		-- get placement if it exists
	place = execute("$"+prefix+"_"+"placementCTRL");
	hipCTRL = execute("$"+prefix+"_hipCTRL;");

		-- Make the starting internal neck/head dummys
	pST = Point pos:ptClav.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
	pST.name = (prefix+"_armStart"+LR+"INT");
	pST.wirecolor = internalColor;

		-- Make a child start, for "tweening" so that the axis of the clav are right.
	pTW = Point pos:ptClav.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(pointSize*0.5);
	pTW.name = (prefix+"_armStartTween"+LR+"INT");
	pTW.wirecolor = internalColor;

		-- Make my bones baby!
    bCL = BoneSys.createBone ptClav.pos ptUpArm.pos [0,0,1]; 			-- clav shoulder bone
	bCL.name = (prefix+"_clav"+LR+"BONE");
	boneSetup bCL true mixColor:sideColor;
    bUA = BoneSys.createBone ptUpArm.pos ptLoArm.pos [0,0,1]; 			-- upper arm bone
	bUA.name = (prefix+"_upArm"+LR+"BONE");
	boneSetup bUA true mixColor:sideColor;
    bLA = BoneSys.createBone ptLoArm.pos ptHand.pos [0,0,1]; 			-- lower arm bone
	bLA.name = (prefix+"_loArm"+LR+"BONE");
	boneSetup bLA true mixColor:sideColor;
		-- override
	bLA.sidefins = bLA.frontfin = bLA.backfin = false;	-- override
	bLA.width = bLA.height = (tipSize/3);	-- make real lo bone small here since only used for IK
	bLA.wirecolor = internalColor;

	-- Make wrist twist bones
	laDir = (ptHand.pos - ptLoArm.pos) / 4.0;
	laStart = ptLoArm.pos;
	laEnd = laStart + laDir;
	bLA1 = BoneSys.createBone laStart laEnd [0,0,1];
	bLA1.name = (prefix+"_loArmTwist1"+LR+"BONE");
	boneSetup bLA1 true mixColor:sideColor;
	laStart = laEnd;
	laEnd = laStart + laDir;
	bLA2 = BoneSys.createBone laStart laEnd [0,0,1];
	bLA2.name = (prefix+"_loArmTwist2"+LR+"BONE");
	boneSetup bLA2 true mixColor:sideColor;
	laStart = laEnd;
	laEnd = laStart + laDir;
	bLA3 = BoneSys.createBone laStart laEnd [0,0,1];
	bLA3.name = (prefix+"_loArmTwist3"+LR+"BONE");
	boneSetup bLA3 true mixColor:sideColor;
	laStart = laEnd;
	laEnd = ptHand.pos;
	bLA4 = BoneSys.createBone laStart laEnd [0,0,1];
	bLA4.name = (prefix+"_loArmTwist4"+LR+"BONE");
	boneSetup bLA4 true mixColor:sideColor;
	-- Set finsize back to proper since they're shorter...
	bLA1.sidefinssize = bLA2.sidefinssize = bLA3.sidefinssize = bLA4.sidefinssize*= 3;
	bLA1.frontfinsize = bLA2.frontfinsize = bLA3.frontfinsize = bLA4.frontfinsize*= 3;
	bLA1.backfinsize = bLA2.backfinsize = bLA3.backfinsize = bLA4.backfinsize *= 3;
/*
	bLA1.wirecolor = (bUA.wirecolor*0.8)+(sideColor*0.2);
	bLA2.wirecolor = (bUA.wirecolor*0.7)+(sideColor*0.3);
	bLA3.wirecolor = (bUA.wirecolor*0.6)+(sideColor*0.4);
	bLA4.wirecolor = (bUA.wirecolor*0.5)+(sideColor*0.5);
*/

    bTP = BoneSys.createBone ptHand.pos (ptHand.pos+[neg*(tipSize/4),0,0]) [0,0,1];	-- lower arm tip bone
	bTP.name = (prefix+"_loArmTip"+LR+"BONE");
	boneSetup bTP false mixColor:sideColor;
		-- Override for tip
	bTP.width = bTP.height = tipSize/4;
	
    bHD = BoneSys.createBone ptHand.pos ptHandTip.pos [0,0,1]; 			-- hand bone
	bHD.name = (prefix+"_hand"+LR+"BONE");
	boneSetup bHD true mixColor:sideColor;
    bHT = BoneSys.createBone ptHandTip.pos (ptHandTip.pos+[neg*(tipSize/4),0,0]) [0,0,1]; 	-- hand tip bone
	bHT.name = (prefix+"_handTip"+LR+"BONE");
	boneSetup bHT false mixColor:sideColor;
		-- Override for Hand tip
	bHT.width = bHT.height = tipSize/4;	 -- special make a bit smaller
																			-- swivel ctrl
	pSWV = Point pos:(ptLoArm.pos+[0,swvDist,0]) isSelected:on centermarker:off axistripod:off cross:on Box:on constantscreensize:off drawontop:off size:pointSize;
	pSWV.name = (prefix+"_elbow"+LR+"CTRL");

	cHand = Ngon radius:(ctrlRad) cornerRadius:0 nsides:6 circular:on scribe:1 pos:ptHand.pos isSelected:on;
	cHand.name = (prefix+"_hand"+LR+"CTRL");
	convertToSplineShape cHand;
		-- Make a 3D CTRL
	cHandCOPY = copy cHand;
	coordsys local (rotate cHandCOPY (eulerangles -90 0 0));	-- rotate
	addAndWeld cHand cHandCOPY 0;

		-- Setup Tween rotation to match first real child...so we zero stuff.
	pST.rotation = bUA.rotation;
	pST.position = ptClav.pos;
	pTW.rotation = bUA.rotation;
	pTW.position = ptClav.pos;

		-- Now link them
	pSWV.parent = hipCTRL;
	zeroOut pSWV;
	pTW.parent = pST;
	bCL.parent = pTW;
	bUA.parent = bCL;
	bLA.parent = bUA;
	bTP.parent = bLA;
	bHD.parent = place;	
	bHT.parent = bHD;

	if (objExists("$"+prefix+"_spine07BONE")) then 
		parST = execute("$"+prefix+"_spine07BONE");
	else if (objExists("$"+prefix+"_spine06BONE")) then 
		parST = execute("$"+prefix+"_spine06BONE");
	else if (objExists("$"+prefix+"_spine05BONE")) then 
		parST = execute("$"+prefix+"_spine05BONE");
	else if (objExists("$"+prefix+"_spine04BONE")) then 
		parST = execute("$"+prefix+"_spine04BONE");
	else if (objExists("$"+prefix+"_spine03BONE")) then 
		parST = execute("$"+prefix+"_spine03BONE");
	else if (objExists("$"+prefix+"_spine02BONE")) then 
		parST = execute("$"+prefix+"_spine02BONE");
	else if (objExists("$"+prefix+"_spine01BONE")) then 
		parST = execute("$"+prefix+"_spine01BONE");
	else if (objExists("$"+prefix+"_hipBONE")) then 
		parST = execute("$"+prefix+"_hipBONE");
	else
		parST = undefined;
	pST.parent = parST;

		-- set some colors
	pSWV.wirecolor = sideColor;
	cHand.wirecolor = sideColor;

		-- Nice controller types...
    cHand.position.controller = Position_XYZ();
    cHand.rotation.controller = Euler_XYZ();
    pSWV.position.controller = Position_XYZ();
    bCL.position.controller = Position_XYZ();
    bLA1.rotation.controller = Euler_XYZ();
    bLA2.rotation.controller = Euler_XYZ();
    bLA3.rotation.controller = Euler_XYZ();
    bLA4.rotation.controller = Euler_XYZ();


		-- Create the IK chain solver/effector's.
	endEf1 = IKSys.ikChain bUA bTP "IKHiSolver";
	endEf1.name = (prefix+"_arm"+LR+"IKEndEffector");
	endEf1.transform.controller.SAParent = 0;
		-- Setup swivel control for knee
	endEf1.transform.controller.VHTarget = pSWV; 
		-- Setup parent for IKeffector
	endEf1.parent = cHand;
		-- For some reason it gets off sometimes, so fix
	endEf1.position = bHD.pos;

		-- Now make our loc/World locators and the one that is constrained between.
	pLC = Point pos:ptHand.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.25*pointSize);
	pLC.name = (prefix+"_local"+LR+"INT");
	pLC.wirecolor = internalColor;
	pWD = Point pos:ptHand.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.50*pointSize);
	pWD.name = (prefix+"_world"+LR+"CTRL");
	pWD.wirecolor = sideColor;
	pLW = Point pos:ptHand.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
	pLW.name = (prefix+"_localWorld"+LR+"INT");
	pLW.wirecolor = internalColor;

	pLC.rotation.controller = Euler_XYZ();
		-- for world locator...make it a LinkConstraint for easier animation locking later
	pWD.transform.controller = Link_Constraint();
	pWD.rotation.controller = Euler_XYZ();

		-- Make rotation of anim ctrls mirrored nicely
	if (LR == "R") then 
		(
		pLC.rotation.y = 180;
		pWD.rotation.y = 180;
		pLC.rotation.x = 180;
		pWD.rotation.x = 180;
		pWD.pos = pLC.pos = ptHand.pos;

		pFX = Point pos:ptHand.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.75*pointSize);
		pFX.name = (prefix+"_parentFixer"+LR+"INT");
		pFX.wirecolor = internalColor;
		pFX.rotation.z = 180;
		pFX.pos = ptHand.pos;
		)

		-- parenting
	pLC.parent = pST;
    if (place != undefined) then
		pWD.transform.controller.AddTarget place 0;	  -- for world, add link, nor parent
	pLW.parent = place;
	cHand.parent = pLW;
	if (pFX != undefined) then
		pFX.parent = cHand;

		-- Now make the localWorld node point constrained between the local and world points.
	pCntl = pLW.pos.controller = position_constraint();
	pCI = pCntl.constraints;
	pCI.appendTarget pLC 50;
	pCI.appendTarget pWD 50;
	pCntl = pLW.rotation.controller = orientation_constraint();
	pCI = pCntl.constraints;
	pCI.appendTarget pLC 50;
	pCI.appendTarget pWD 50;
	    
		-- Add CA's to hand
	addArmAttrs(cHand);

		-- Do stretchy stuff if needed
	if (stretchyArms == true) then 
		makeStretchyBones bUA bLA bTP endEf1 cHand ("$"+cHand.name+" $"+pST.name+" ");

		-- Now we gotta hook up the localWorld attrs to drive the constraint made above.
    if (EmptyModifier != undefined) then 
		cntHD = cHand.Attribute_Holder.ArmCustom_Attributes;
	else
		cntHD = cHand.ArmCustom_Attributes;
	paramWire.connect cntHD[#localworld] pLW.position.controller[1] "100-localworld;";
	paramWire.connect cntHD[#localworld] pLW.position.controller[2] "localworld;";
	paramWire.connect cntHD[#localworld] pLW.rotation.controller[1] "100-localworld;";
	paramWire.connect cntHD[#localworld] pLW.rotation.controller[2] "localworld;";

		-- Parent wrist twist bones
	bLA1.parent = bLA;
	bLA2.parent = bLA;
	bLA3.parent = bLA;
	bLA4.parent = bLA;

		-- Also setup wristTwist bones
	paramWire.connect cntHD[#wristtwist] bLA1.rotation.controller[1] ("degToRad(wristtwist)*0.07*"+(neg as string)+";");
	paramWire.connect cntHD[#wristtwist] bLA2.rotation.controller[1] ("degToRad(wristtwist)*0.25*"+(neg as string)+";");
	paramWire.connect cntHD[#wristtwist] bLA3.rotation.controller[1] ("degToRad(wristtwist)*0.75*"+(neg as string)+";");
	paramWire.connect cntHD[#wristtwist] bLA4.rotation.controller[1] ("degToRad(wristtwist)*1.00*"+(neg as string)+";");


		-- Now we setup a script controller on the wrist twists so that the
		-- wrist twist bones stay in the right place when stretchy stuff happens
	if (stretchyArms == true) then
		( 
		makeStretchyTwist 1 neg bLA1 bLA bUA endEf1 cHand ("$"+cHand.name+" $"+pST.name+" ");
		makeStretchyTwist 2 neg bLA2 bLA bUA endEf1 cHand ("$"+cHand.name+" $"+pST.name+" ");
		makeStretchyTwist 3 neg bLA3 bLA bUA endEf1 cHand ("$"+cHand.name+" $"+pST.name+" ");
		makeStretchyTwist 4 neg bLA4 bLA bUA endEf1 cHand ("$"+cHand.name+" $"+pST.name+" ");
		)

	
		-- Now we gotta make the hand bone stuck to the end of the arm,
		-- but rotated based on our hand control.
	pCntl = bHD.pos.controller = position_constraint();
	pCI = pCntl.constraints;
	pCI.appendTarget bTP 100;
	if (pFX == undefined) then
		(
		pCntl = bHD.rotation.controller = orientation_constraint();
		pCI = pCntl.constraints;
		pCI.appendTarget cHand 50;
		)
	else (
		pCntl = bHD.rotation.controller = orientation_constraint();
		pCI = pCntl.constraints;
		pCI.appendTarget pFX 50;
		)

		-- Now we do wiring for reach and shrug onto clavicle
	paramWire.connect cntHD[#reach] bCL.position.controller[2] ("reach / -100.0 * "+((ctrlRad) as string)+";");
	paramWire.connect cntHD[#shrug] bCL.position.controller[3] ("shrug / 100.0 * "+((ctrlRad) as string)+";");
	
	format "% Arm Created.\n" LR;
	return 1;
)


-- -----------------------------------------------------------------------------------------------


/*
 * createToes() - Makes a R or L toe setup
 */
fn createToes LR =
(

		-- Setup variables for the proper side
	if (LR == "L") then
		(
		ptToe11 = $ptToe1L1;
		ptToe12 = $ptToe1L2;
		ptToe13 = $ptToe1L3;
		ptToe14 = $ptToe1L4;
		ptToeUp = $ptToeLUp;
		ptToe21 = $ptToe2L1;
		ptToe22 = $ptToe2L2;
		ptToe23 = $ptToe2L3;
		ptToe24 = $ptToe2L4;
		ptToe31 = $ptToe3L1;
		ptToe32 = $ptToe3L2;
		ptToe33 = $ptToe3L3;
		ptToe34 = $ptToe3L4;
		ptToe41 = $ptToe4L1;
		ptToe42 = $ptToe4L2;
		ptToe43 = $ptToe4L3;
		ptToe44 = $ptToe4L4;
		ptToe51 = $ptToe5L1;
		ptToe52 = $ptToe5L2;
		ptToe53 = $ptToe5L3;
		ptToe54 = $ptToe5L4;
		neg = 1;
		sideColor = 0.75*leftColor;
		)
	else if (LR == "R") then
		(
		ptToe11 = $ptToe1R1;
		ptToe12 = $ptToe1R2;
		ptToe13 = $ptToe1R3;
		ptToe14 = $ptToe1R4;
		ptToeUp = $ptToeRUp;
		ptToe21 = $ptToe2R1;
		ptToe22 = $ptToe2R2;
		ptToe23 = $ptToe2R3;
		ptToe24 = $ptToe2R4;
		ptToe31 = $ptToe3R1;
		ptToe32 = $ptToe3R2;
		ptToe33 = $ptToe3R3;
		ptToe34 = $ptToe3R4;
		ptToe41 = $ptToe4R1;
		ptToe42 = $ptToe4R2;
		ptToe43 = $ptToe4R3;
		ptToe44 = $ptToe4R4;
		ptToe51 = $ptToe5R1;
		ptToe52 = $ptToe5R2;
		ptToe53 = $ptToe5R3;
		ptToe54 = $ptToe5R4;
		neg = -1;
		sideColor = 0.75*rightColor;
		)
	else if (LR == "L2") then
		(
		ptToe11 = $ptToe1L21;
		ptToe12 = $ptToe1L22;
		ptToe13 = $ptToe1L23;
		ptToe14 = $ptToe1L24;
		ptToeUp = $ptToeL2Up;
		ptToe21 = $ptToe2L21;
		ptToe22 = $ptToe2L22;
		ptToe23 = $ptToe2L23;
		ptToe24 = $ptToe2L24;
		ptToe31 = $ptToe3L21;
		ptToe32 = $ptToe3L22;
		ptToe33 = $ptToe3L23;
		ptToe34 = $ptToe3L24;
		ptToe41 = $ptToe4L21;
		ptToe42 = $ptToe4L22;
		ptToe43 = $ptToe4L23;
		ptToe44 = $ptToe4L24;
		ptToe51 = $ptToe5L21;
		ptToe52 = $ptToe5L22;
		ptToe53 = $ptToe5L23;
		ptToe54 = $ptToe5L24;
		neg = 1;
		sideColor = 0.75*leftColor;
		)
	else
		(
		ptToe11 = $ptToe1R21;
		ptToe12 = $ptToe1R22;
		ptToe13 = $ptToe1R23;
		ptToe14 = $ptToe1R24;
		ptToeUp = $ptToeR2Up;
		ptToe21 = $ptToe2R21;
		ptToe22 = $ptToe2R22;
		ptToe23 = $ptToe2R23;
		ptToe24 = $ptToe2R24;
		ptToe31 = $ptToe3R21;
		ptToe32 = $ptToe3R22;
		ptToe33 = $ptToe3R23;
		ptToe34 = $ptToe3R24;
		ptToe41 = $ptToe4R21;
		ptToe42 = $ptToe4R22;
		ptToe43 = $ptToe4R23;
		ptToe44 = $ptToe4R24;
		ptToe51 = $ptToe5R21;
		ptToe52 = $ptToe5R22;
		ptToe53 = $ptToe5R23;
		ptToe54 = $ptToe5R24;
		neg = -1;
		sideColor = 0.75*rightColor;
		)

	cFoot = execute("$"+prefix+"_foot"+LR+"CTRL;");

	if (cFoot == undefined) then 
		(
		format "WARNING: Can't create % Toes.  Foot Control must exist first.\n" LR;
		return 0;
		)

	bFT = execute("$"+prefix+"_foot"+LR+"BONE;");
	bTO = execute("$"+prefix+"_toe"+LR+"BONE;");

	if (bFT == undefined or bTO == undefined) then 
		(
		format "WARNING: Can't create % Toes.  Foot and Main Toe Bone must exist first.\n" LR;
		return 0;
		)

	-- pre calc # of joints and such so that we can add attrs
	doToe1 = false;
	numToes = 0;
    if (ptToeUp != undefined and ptToe11 != undefined and ptToe12 != undefined	) then 
		doToe1 = true;
    if (ptToe21 != undefined and ptToe22 != undefined	) then 
		(
		numToes += 1;
	    if (ptToe31 != undefined and ptToe32 != undefined	) then 
			(
			numToes += 1;
		    if (ptToe41 != undefined and ptToe42 != undefined	) then 
				(
				numToes += 1;
			    if (ptToe51 != undefined and ptToe52 != undefined	) then 
					(
					numToes += 1;
					)
				)
			)
		)
	
		-- Add CA's to ctrl object!
    if (doToe1 == true or numToes > 0) then
	    addToeAttrs cFoot doToe1 numToes;
	else
		return;

		
	bLast = undefined;		-- last bone
	lastP = 0;				-- last end position

	toeDigits = 0;

    if ((objExists("$"+prefix+"_toe"+LR+"1CTRL") != true) and 
		ptToeUp != undefined and ptToe11 != undefined and ptToe12 != undefined	) then 
		(
		format "Building % Toe 1.\n" LR;

		tUp = ptToeUp.pos - ptToe11.pos;
		ctrlRad = length(ptToe11.pos - ptToe12.pos)*0.75;
		tipSize = 0.17 * (length(ptToe12.pos - ptToe11.pos));
		pointSize = tipSize*4;


	    bT1 = BoneSys.createBone ptToe11.pos ptToe12.pos tUp;
		bT1.name = (prefix+"_toe1"+LR+"1BONE");
		boneSetup bT1 true mixColor:sideColor;
		bT1.rotation.controller = Euler_XYZ();
		bLast = bT1;			-- keep track so we can make tip
		lastP = ptToe12.pos;
		toeDigits += 1;
		cT1 = Ngon radius:(ctrlRad*1.2) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptToe11.pos isSelected:on;
		cT1.name = (prefix+"_toe1"+LR+"1CTRL");
		cT1.wirecolor = sideColor;
		convertToSplineShape cT1;
		cT1.rotation = bT1.rotation;		-- get control to right orientation
		coordsys local (rotate cT1 (eulerangles -90 0 0));
		cT1.position = bT1.position;
		cT1.rotation.controller = Euler_XYZ();


	    if (ptToe12 != undefined and ptToe13 != undefined	) then 
			(
	    	bT2 = BoneSys.createBone ptToe12.pos ptToe13.pos tUp;
			bT2.name = (prefix+"_toe1"+LR+"2BONE");
			boneSetup bT2 true mixColor:sideColor;
			bT2.rotation.controller = Euler_XYZ();
			bLast = bT2;			-- keep track so we can make tip
			lastP = ptToe13.pos;
			toeDigits += 1;
			cT2 = Ngon radius:(ctrlRad*1.1) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptToe12.pos isSelected:on;
			cT2.name = (prefix+"_toe1"+LR+"2CTRL");
			cT2.wirecolor = sideColor;
			convertToSplineShape cT2;
			cT2.rotation = bT2.rotation;		-- get control to right orientation
			coordsys local (rotate cT2 (eulerangles -90 0 0));
			cT2.position = bT2.position;
			cT2.rotation.controller = Euler_XYZ();

		    if (ptToe13 != undefined and ptToe14 != undefined	) then 
				(
		    	bT3 = BoneSys.createBone ptToe13.pos ptToe14.pos tUp;
				bT3.name = (prefix+"_toe1"+LR+"3BONE");
				boneSetup bT3 true mixColor:sideColor;
				bT3.rotation.controller = Euler_XYZ();
				bLast = bT3;			-- keep track so we can make tip
				lastP = ptToe14.pos;
				toeDigits += 1;
				cT3 = Ngon radius:(ctrlRad*1.0) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptToe13.pos isSelected:on;
				cT3.name = (prefix+"_toe1"+LR+"3CTRL");
				cT3.wirecolor = sideColor;
				convertToSplineShape cT3;
				cT3.rotation = bT3.rotation;		-- get control to right orientation
				coordsys local (rotate cT3 (eulerangles -90 0 0));
				cT3.position = bT3.position;
				cT3.rotation.controller = Euler_XYZ();

				)	
			)

		-- Now make tip bone
		bTP = BoneSys.createBone lastP ((0.5*(ptToe12.pos-ptToe11.pos))+lastP) tUp;
		bTP.name = (prefix+"_toe1"+LR+"TipBONE");
		boneSetup bTP false;
			-- override
		bTP.width = bTP.height = tipSize*3;

		-- Make Toe1 Adjuster Dummy and Toe1 Master
		pTM = Point pos:ptToe11.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
		pTM.name = (prefix+"_toe1"+LR+"MasterINT");
		pTM.wirecolor = internalColor;
		pTM.rotation = bT1.rotation;
		pTM.position = bT1.pos;
		
		pTA = Point pos:ptToe11.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
		pTA.name = (prefix+"_toe1"+LR+"AdjusterINT");
		pTA.rotation.controller = Euler_XYZ();
		pTA.wirecolor = internalColor;
		pTA.rotation = bT1.rotation;
		pTA.position = bT1.pos;

		-- Parent stuff
--		pTM.parent = bFT;		-- parent to Bone for foot for the first toe.  All others are on the toe.?
		pTM.parent = bTO;	    -- ah most people will prob want the main toe..so do that...
		pTA.parent = pTM;
		cT1.parent = pTA
		bT1.parent = cT1;
		bTP.parent = bT1;
		zeroOut cT1;
		if (cT2 != undefined) then
			(
			cT2.parent = bT1;
			bT2.parent = cT2;
			bTP.parent = bT2;
			zeroOut cT2;
			)
		if (cT3 != undefined) then
			(
			cT3.parent = bT2;
			bT3.parent = cT3;
			bTP.parent = bT3;
			zeroOut cT3;
			)

		-- At this point things are all made and linked for the Toe1...
		-- we need to set up the auto rotations by wiring from the attrs.

--	    format "WIRING Toe1...\n";
	    if (EmptyModifier != undefined) then 
			cntFT = cFoot.Attribute_Holder.ToeCustom_Attributes;
		else
			cntFT = cFoot.ToeCustom_Attributes;
--		format "cntFT = %\n" cntFT;
		
		paramWire.connect cntFT[#toe1] bT1.rotation.controller[3] ("toe1/100.0*"+(tDig1Mult as string)+"*"+((degToRad(360)/toeDigits) as string));
		if (bT2 != undefined) then 
			paramWire.connect cntFT[#toe1] bT2.rotation.controller[3] ("toe1/100.0*"+(tDig2Mult as string)+"*"+((degToRad(360)/toeDigits) as string));
		if (bT3 != undefined) then 
			paramWire.connect cntFT[#toe1] bT3.rotation.controller[3] ("toe1/100.0*"+(tDig3Mult as string)+"*"+((degToRad(360)/toeDigits) as string));
		paramWire.connect cntFT[#toe1adj] pTA.rotation.controller[3] ("toe1adj/100.0*"+((neg*degToRad(90)) as string));
		paramWire.connect cntFT[#toe1twist] pTA.rotation.controller[1] ("toe1twist/100.0*"+((neg*degToRad(90)) as string));
--	    format "Toe1 wired.\n";


		)
	else (
		format "WARNING: Can't create % Toe.  pointHelpers are missing or Toe1 already Exists.\n" LR;
		)

		--
		-- Now Make Da FINGERS
		--
	for f in 2 to (numToes+1) do
		(
		toeDigits = 0;
		bF1 = bF2 = bF3 = undefined;
		cF1 = cF2 = cF3 = undefined;
		bFP = pFM = pFA = undefined;
		
			-- Make easy variables based on which finger we're on.
		ptToe1 = execute("$ptToe"+(f as string)+LR+"1;");
		ptToe2 = execute("$ptToe"+(f as string)+LR+"2;");
		ptToe3 = execute("$ptToe"+(f as string)+LR+"3;");
		ptToe4 = execute("$ptToe"+(f as string)+LR+"4;");

		format "Building toe %%.\n" f LR;
		
	    if (objExists("$"+prefix+"_toe"+(f as string)+LR+"1CTRL") != true and 
			ptToe1 != undefined and ptToe2 != undefined	) then 
			(
			fUp = [0,0,1];
			ctrlRad = length(ptToe1.pos - ptToe2.pos)*0.75;
			tipSize = 0.17 * (length(ptToe2.pos - ptToe1.pos));
			pointSize = tipSize*4;
	

	
		    bF1 = BoneSys.createBone ptToe1.pos ptToe2.pos fUp;
			bF1.name = (prefix+"_toe"+(f as string)+LR+"1BONE");
			boneSetup bF1 true mixColor:sideColor;
			bF1.rotation.controller = Euler_XYZ();
			bLast = bF1;			-- keep track so we can make tip
			lastP = ptToe2.pos;
			toeDigits += 1;
			cF1 = Ngon radius:(ctrlRad*1.2) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptToe1.pos isSelected:on;
			cF1.name = (prefix+"_toe"+(f as string)+LR+"1CTRL");
			cF1.wirecolor = sideColor;
			convertToSplineShape cF1;
			cF1.rotation = bF1.rotation;		-- get control to right orientation
			coordsys local (rotate cF1 (eulerangles -90 0 0));
			cF1.position = bF1.position;
			cF1.rotation.controller = Euler_XYZ();


		    if (ptToe2 != undefined and ptToe3 != undefined	) then 
				(
		    	bF2 = BoneSys.createBone ptToe2.pos ptToe3.pos fUp;
				bF2.name = (prefix+"_toe"+(f as string)+LR+"2BONE");
				boneSetup bF2 true mixColor:sideColor;
				bF2.rotation.controller = Euler_XYZ();
				bLast = bF2;			-- keep track so we can make tip
				lastP = ptToe3.pos;
				toeDigits+= 1;
				cF2 = Ngon radius:(ctrlRad*1.1) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptToe2.pos isSelected:on;
				cF2.name = (prefix+"_toe"+(f as string)+LR+"2CTRL");
				cF2.wirecolor = sideColor;
				convertToSplineShape cF2;
				cF2.rotation = bF2.rotation;		-- get control to right orientation
				coordsys local (rotate cF2 (eulerangles -90 0 0));
				cF2.position = bF2.position;
				cF2.rotation.controller = Euler_XYZ();
	
			    if (ptToe3 != undefined and ptToe4 != undefined	) then 
					(
			    	bF3 = BoneSys.createBone ptToe3.pos ptToe4.pos fUp;
					bF3.name = (prefix+"_toe"+(f as string)+LR+"3BONE");
					boneSetup bF3 true mixColor:sideColor;
					bF3.rotation.controller = Euler_XYZ();
					bLast = bF3;			-- keep track so we can make tip
					lastP = ptToe4.pos;
					toeDigits+= 1;
					cF3 = Ngon radius:(ctrlRad*1.0) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptToe3.pos isSelected:on;
					cF3.name = (prefix+"_toe"+(f as string)+LR+"3CTRL");
					cF3.wirecolor = sideColor;
					convertToSplineShape cF3;
					cF3.rotation = bF3.rotation;		-- get control to right orientation
					coordsys local (rotate cF3 (eulerangles -90 0 0));
					cF3.position = bF3.position;
					cF3.rotation.controller = Euler_XYZ();
	
					)	
				)


			-- Now make tip bone
			bFP = BoneSys.createBone lastP ((0.5*(ptToe2.pos-ptToe1.pos))+lastP) fUp;
			bFP.name = (prefix+"_toe"+(f as string)+LR+"TipBONE");
			boneSetup bFP false;
				-- override
			bFP.width = bFP.height = tipSize*3;


			-- Make Finger Master dummy and Finger Adjuster dummy
			pFM = Point pos:ptToe1.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
			pFM.name = (prefix+"_toe"+(f as string)+LR+"MasterINT");
			pFM.wirecolor = internalColor;
			pFM.rotation = bF1.rotation;
			pFM.position = bF1.pos;
	
			pFA = Point pos:ptToe1.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
			pFA.name = (prefix+"_toe"+(f as string)+LR+"AdjusterINT");
			pFA.rotation.controller = Euler_XYZ();
			pFA.wirecolor = internalColor;
			pFA.rotation = bF1.rotation;
			pFA.position = bF1.pos;
	
			-- Parent stuff
			pFM.parent = bTO;		-- parent to Bone for master regular large toe which already does footRoll.
			pFA.parent = pFM;
			cF1.parent = pFA
			bF1.parent = cF1;
			bFP.parent = bF1;
			zeroOut cF1;
			if (cF2 != undefined) then
				(
				cF2.parent = bF1;		-- control is parented to previous bone
				bF2.parent = cF2;		-- bone is parented to this control
				bFP.parent = bF2;
				zeroOut cF2;
				)
			if (cF3 != undefined) then
				(
				cF3.parent = bF2;
				bF3.parent = cF3;
				bFP.parent = bF3;
				zeroOut cF3;
				)
 

			-- At this point things are all made and linked for the finger...
			-- we need to set up the auto rotations by wiring from the attrs.
		    if (EmptyModifier != undefined) then 
				cntFT = "$"+cFoot.name+".Attribute_Holder.ToeCustom_Attributes";
			else
				cntFT = "$"+cFoot.name+".ToeCustom_Attributes";
	
			format "Wiring %...\n" f;
			str = "paramWire.connect "+cntFT+"[#toe"+(f as string)+"] $"+bF1.name+".rotation.controller[3] \"toe"+(f as string)+"/100.0*"+(tDig1Mult as string)+"*"+((degToRad(360)/toeDigits) as string)+"\";\n";
			if (bF2 != undefined) then 
				str += "paramWire.connect "+cntFT+"[#toe"+(f as string)+"] $"+bF2.name+".rotation.controller[3] \"toe"+(f as string)+"/100.0*"+(tDig2Mult as string)+"*"+((degToRad(360)/toeDigits) as string)+"\";\n";
			if (bF3 != undefined) then 
				str += "paramWire.connect "+cntFT+"[#toe"+(f as string)+"] $"+bF3.name+".rotation.controller[3] \"toe"+(f as string)+"/100.0*"+(tDig3Mult as string)+"*"+((degToRad(360)/toeDigits) as string)+"\";\n";
			execute str;			
			format "%% wired.\n" f LR;
	
			-- Finally take care of spread if needed
		    if (numToes > 1) then
				(
				deg = 0;
				if (f-1 <= (numToes/2)) then
					deg = degToRad(-10);
				else
					deg = degToRad(10);
						-- outer toes do more...
				if (f == 2) then
					deg -= degToRad(20);
				if (f-1 == numToes) then
					deg += degToRad(20);
--				format "Doing spread %...\n" f;
				str = "paramWire.connect "+cntFT+"[#toespread] $'"+pFA.name+"'.rotation.controller[3] \"toespread/100.0*"+(deg as string)+"\";\n";
				execute str;
--				format "Spread %% done.\n" f LR;
				)		
	
			)
		else (
			format "WARNING: Can't create Toe %%.  pointHelpers are missing or Toe already Exists.\n" f LR;
			)

		) -- end of numToe loop

	format "% Toes created.\n" LR;
    return 1;
)


-- -----------------------------------------------------------------------------------------------


/*
 * createFingers() - Makes a R or L finger setup
 */
fn createFingers LR =
(

		-- Setup variables for the proper side
	if (LR == "L") then
		(
		ptThumb1 = $ptThumbL1;
		ptThumb2 = $ptThumbL2;
		ptThumb3 = $ptThumbL3;
		ptThumb4 = $ptThumbL4;
		ptThumbUp = $ptThumbLUp;
		ptFinger11 = $ptFinger1L1;
		ptFinger12 = $ptFinger1L2;
		ptFinger13 = $ptFinger1L3;
		ptFinger14 = $ptFinger1L4;
		ptFinger21 = $ptFinger2L1;
		ptFinger22 = $ptFinger2L2;
		ptFinger23 = $ptFinger2L3;
		ptFinger24 = $ptFinger2L4; 
		ptFinger31 = $ptFinger3L1;
		ptFinger32 = $ptFinger3L2;
		ptFinger33 = $ptFinger3L3;
		ptFinger34 = $ptFinger3L4;
		ptFinger41 = $ptFinger4L1;
		ptFinger42 = $ptFinger4L2;
		ptFinger43 = $ptFinger4L3;
		ptFinger44 = $ptFinger4L4;
		neg = 1;
		sideColor = 0.75*leftColor;
		)
	else (
		ptThumb1 = $ptThumbR1;
		ptThumb2 = $ptThumbR2;
		ptThumb3 = $ptThumbR3;
		ptThumb4 = $ptThumbR4;
		ptThumbUp = $ptThumbRUp;
		ptFinger11 = $ptFinger1R1;
		ptFinger12 = $ptFinger1R2;
		ptFinger13 = $ptFinger1R3;
		ptFinger14 = $ptFinger1R4;
		ptFinger21 = $ptFinger2R1;
		ptFinger22 = $ptFinger2R2;
		ptFinger23 = $ptFinger2R3;
		ptFinger24 = $ptFinger2R4; 
		ptFinger31 = $ptFinger3R1;
		ptFinger32 = $ptFinger3R2;
		ptFinger33 = $ptFinger3R3;
		ptFinger34 = $ptFinger3R4;
		ptFinger41 = $ptFinger4R1;
		ptFinger42 = $ptFinger4R2;
		ptFinger43 = $ptFinger4R3;
		ptFinger44 = $ptFinger4R4;
		neg = -1;
		sideColor = 0.75*rightColor;
		)

	cHand = execute("$"+prefix+"_hand"+LR+"CTRL;");
	if (cHand == undefined) then 
		(
		format "WARNING: Can't create % Fingers.  Hand Control must exist first.\n" LR;
		return 0;
		)
    bHand = execute("$"+prefix+"_hand"+LR+"BONE;");
	if (bHand == undefined) then 
		(
		format "WARNING: Can't create % Fingers.  Hand Bone must exist first.\n" LR;
		return 0;
		)
		
	-- pre calc # of joints and such so that we can add attrs
	doThumb = false;
	numFingers = 0;
    if (ptThumbUp != undefined and ptThumb1 != undefined and ptThumb2 != undefined	) then 
		doThumb = true;
    if (ptFinger11 != undefined and ptFinger12 != undefined	) then 
		(
		numFingers += 1;
	    if (ptFinger21 != undefined and ptFinger22 != undefined	) then 
			(
			numFingers += 1;
		    if (ptFinger31 != undefined and ptFinger32 != undefined	) then 
				(
				numFingers += 1;
			    if (ptFinger41 != undefined and ptFinger42 != undefined	) then 
					(
					numFingers += 1;
					)
				)
			)
		)

		-- Add CA's to ctrl object!
    if (doThumb == true or numFingers > 0) then
	    addFingerAttrs cHand doThumb numFingers;
	else
	    return;
		
	bLast = undefined;		-- last bone
	lastP = 0;				-- last end position

	thumbDigits = 0;

    if ((objExists("$"+prefix+"_thumb"+LR+"1CTRL") != true) and 
		ptThumbUp != undefined and ptThumb1 != undefined and ptThumb2 != undefined	) then 
		(
		format "Building % Thumb.\n" LR;

		tUp = ptThumbUp.pos - ptThumb1.pos;
		ctrlRad = length(ptThumb1.pos - ptThumb2.pos)*0.75;
		tipSize = 0.17 * (length(ptThumb2.pos - ptThumb1.pos));
		pointSize = tipSize*4;


	    bT1 = BoneSys.createBone ptThumb1.pos ptThumb2.pos tUp;
		bT1.name = (prefix+"_thumb"+LR+"1BONE");
		boneSetup bT1 true mixColor:sideColor;
		bT1.rotation.controller = Euler_XYZ();
		bLast = bT1;			-- keep track so we can make tip
		lastP = ptThumb2.pos;
		thumbDigits += 1;
		cT1 = Ngon radius:(ctrlRad*1.2) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptThumb1.pos isSelected:on;
		cT1.name = (prefix+"_thumb"+LR+"1CTRL");
		cT1.wirecolor = sideColor;
		convertToSplineShape cT1;
		cT1.rotation = bT1.rotation;		-- get control to right orientation
		coordsys local (rotate cT1 (eulerangles -90 0 0));
		cT1.position = bT1.position;
		cT1.rotation.controller = Euler_XYZ();


	    if (ptThumb2 != undefined and ptThumb3 != undefined	) then 
			(
	    	bT2 = BoneSys.createBone ptThumb2.pos ptThumb3.pos tUp;
			bT2.name = (prefix+"_thumb"+LR+"2BONE");
			boneSetup bT2 true mixColor:sideColor;
			bT2.rotation.controller = Euler_XYZ();
			bLast = bT2;			-- keep track so we can make tip
			lastP = ptThumb3.pos;
			thumbDigits += 1;
			cT2 = Ngon radius:(ctrlRad*1.1) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptThumb2.pos isSelected:on;
			cT2.name = (prefix+"_thumb"+LR+"2CTRL");
			cT2.wirecolor = sideColor;
			convertToSplineShape cT2;
			cT2.rotation = bT2.rotation;		-- get control to right orientation
			coordsys local (rotate cT2 (eulerangles -90 0 0));
			cT2.position = bT2.position;
			cT2.rotation.controller = Euler_XYZ();

		    if (ptThumb3 != undefined and ptThumb4 != undefined	) then 
				(
		    	bT3 = BoneSys.createBone ptThumb3.pos ptThumb4.pos tUp;
				bT3.name = (prefix+"_thumb"+LR+"3BONE");
				boneSetup bT3 true mixColor:sideColor;
				bT3.rotation.controller = Euler_XYZ();
				bLast = bT3;			-- keep track so we can make tip
				lastP = ptThumb4.pos;
				thumbDigits += 1;
				cT3 = Ngon radius:(ctrlRad*1.0) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptThumb3.pos isSelected:on;
				cT3.name = (prefix+"_thumb"+LR+"3CTRL");
				cT3.wirecolor = sideColor;
				convertToSplineShape cT3;
				cT3.rotation = bT3.rotation;		-- get control to right orientation
				coordsys local (rotate cT3 (eulerangles -90 0 0));
				cT3.position = bT3.position;
				cT3.rotation.controller = Euler_XYZ();

				)	
			)

		-- Now make tip bone
		bTP = BoneSys.createBone lastP ((0.5*(ptThumb2.pos-ptThumb1.pos))+lastP) tUp;
		bTP.name = (prefix+"_thumb"+LR+"TipBONE");
		boneSetup bTP false;
			-- override
		bTP.width = bTP.height = tipSize*3;

		-- Make thumb Adjuster Dummy and Thumb Master
		pTM = Point pos:ptThumb1.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
		pTM.name = (prefix+"_thumb"+LR+"MasterINT");
		pTM.wirecolor = internalColor;
		pTM.rotation = bT1.rotation;
		pTM.position = bT1.pos;
		
		pTA = Point pos:ptThumb1.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
		pTA.name = (prefix+"_thumb"+LR+"AdjusterINT");
		pTA.rotation.controller = Euler_XYZ();
		pTA.wirecolor = internalColor;
		pTA.rotation = bT1.rotation;
		pTA.position = bT1.pos;

		-- Parent stuff
		pTM.parent = bHand;		-- parent to Bone which is stuck on end of arm
		pTA.parent = pTM;
		cT1.parent = pTA
		bT1.parent = cT1;
		bTP.parent = bT1;
		zeroOut cT1;
		if (cT2 != undefined) then
			(
			cT2.parent = bT1;
			bT2.parent = cT2;
			bTP.parent = bT2;
			zeroOut cT2;
			)
		if (cT3 != undefined) then
			(
			cT3.parent = bT2;
			bT3.parent = cT3;
			bTP.parent = bT3;
			zeroOut cT3;
			)

		-- At this point things are all made and linked for the thumb...
		-- we need to set up the auto rotations by wiring from the attrs.

--	    format "WIRING Thumb...\n";
	    if (EmptyModifier != undefined) then 
			cntHD = cHand.Attribute_Holder.Custom_Attributes;
		else
			cntHD = cHand.Custom_Attributes;
--		format "cntHD = %\n" cntHD;
		
		paramWire.connect cntHD[#thumb] bT1.rotation.controller[3] ("thumb/100.0*"+(fDig1Mult as string)+"*"+((degToRad(360)/thumbDigits) as string));
		if (bT2 != undefined) then 
			paramWire.connect cntHD[#thumb] bT2.rotation.controller[3] ("thumb/100.0*"+(fDig2Mult as string)+"*"+((degToRad(360)/thumbDigits) as string));
		if (bT3 != undefined) then 
			paramWire.connect cntHD[#thumb] bT3.rotation.controller[3] ("thumb/100.0*"+(fDig3Mult as string)+"*"+((degToRad(360)/thumbDigits) as string));
		paramWire.connect cntHD[#thumbadj] pTA.rotation.controller[3] ("thumbadj/100.0*"+((neg*degToRad(90)) as string));
		paramWire.connect cntHD[#thumbtwist] pTA.rotation.controller[1] ("thumbtwist/100.0*"+((neg*degToRad(90)) as string));
--	    format "Thumb wired.\n";


		)
	else (
		format "WARNING: Can't create % Thumb.  pointHelpers are missing or Thumb already Exists.\n" LR;
		)

		--
		-- Now Make Da FINGERS
		--
	for f in 1 to numFingers do
		(
		fingerDigits = 0;
		bF1 = bF2 = bF3 = undefined;
		cF1 = cF2 = cF3 = undefined;
		bFP = pFM = pFA = undefined;
		
			-- Make easy variables based on which finger we're on.
		ptFinger1 = execute("$ptFinger"+(f as string)+LR+"1;");
		ptFinger2 = execute("$ptFinger"+(f as string)+LR+"2;");
		ptFinger3 = execute("$ptFinger"+(f as string)+LR+"3;");
		ptFinger4 = execute("$ptFinger"+(f as string)+LR+"4;");

		format "Building finger %%.\n" f LR;
		
	    if (objExists("$"+prefix+"_finger"+(f as string)+LR+"1CTRL") != true and 
			ptFinger1 != undefined and ptFinger2 != undefined	) then 
			(
--			fUp = cross(ptFinger2.pos - ptFinger1.pos);
			fUp = [0,0,1];
			ctrlRad = length(ptFinger1.pos - ptFinger2.pos)*0.75;
			tipSize = 0.17 * (length(ptFinger2.pos - ptFinger1.pos));
			pointSize = tipSize*4;
	

	
		    bF1 = BoneSys.createBone ptFinger1.pos ptFinger2.pos fUp;
			bF1.name = (prefix+"_finger"+(f as string)+LR+"1BONE");
			boneSetup bF1 true mixColor:sideColor;
			bF1.rotation.controller = Euler_XYZ();
			bLast = bF1;			-- keep track so we can make tip
			lastP = ptFinger2.pos;
			fingerDigits += 1;
			cF1 = Ngon radius:(ctrlRad*1.2) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptFinger1.pos isSelected:on;
			cF1.name = (prefix+"_finger"+(f as string)+LR+"1CTRL");
			cF1.wirecolor = sideColor;
			convertToSplineShape cF1;
			cF1.rotation = bF1.rotation;		-- get control to right orientation
			coordsys local (rotate cF1 (eulerangles -90 0 0));
			cF1.position = bF1.position;
			cF1.rotation.controller = Euler_XYZ();


		    if (ptFinger2 != undefined and ptFinger3 != undefined	) then 
				(
		    	bF2 = BoneSys.createBone ptFinger2.pos ptFinger3.pos fUp;
				bF2.name = (prefix+"_finger"+(f as string)+LR+"2BONE");
				boneSetup bF2 true mixColor:sideColor;
				bF2.rotation.controller = Euler_XYZ();
				bLast = bF2;			-- keep track so we can make tip
				lastP = ptFinger3.pos;
				fingerDigits += 1;
				cF2 = Ngon radius:(ctrlRad*1.1) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptFinger2.pos isSelected:on;
				cF2.name = (prefix+"_finger"+(f as string)+LR+"2CTRL");
				cF2.wirecolor = sideColor;
				convertToSplineShape cF2;
				cF2.rotation = bF2.rotation;		-- get control to right orientation
				coordsys local (rotate cF2 (eulerangles -90 0 0));
				cF2.position = bF2.position;
				cF2.rotation.controller = Euler_XYZ();
	
			    if (ptFinger3 != undefined and ptFinger4 != undefined	) then 
					(
			    	bF3 = BoneSys.createBone ptFinger3.pos ptFinger4.pos fUp;
					bF3.name = (prefix+"_finger"+(f as string)+LR+"3BONE");
					boneSetup bF3 true mixColor:sideColor;
					bF3.rotation.controller = Euler_XYZ();
					bLast = bF3;			-- keep track so we can make tip
					lastP = ptFinger4.pos;
					fingerDigits += 1;
					cF3 = Ngon radius:(ctrlRad*1.0) cornerRadius:0 nsides:6 circular:off scribe:1 pos:ptFinger3.pos isSelected:on;
					cF3.name = (prefix+"_finger"+(f as string)+LR+"3CTRL");
					cF3.wirecolor = sideColor;
					convertToSplineShape cF3;
					cF3.rotation = bF3.rotation;		-- get control to right orientation
					coordsys local (rotate cF3 (eulerangles -90 0 0));
					cF3.position = bF3.position;
					cF3.rotation.controller = Euler_XYZ();
	
					)	
				)


			-- Now make tip bone
			bFP = BoneSys.createBone lastP ((0.5*(ptFinger2.pos-ptFinger1.pos))+lastP) fUp;
			bFP.name = (prefix+"_finger"+(f as string)+LR+"TipBONE");
			boneSetup bFP false;
				-- override
			bFP.width = bFP.height = tipSize*3;


			-- Make Finger Master dummy and Finger Adjuster dummy
			pFM = Point pos:ptFinger1.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:pointSize;
			pFM.name = (prefix+"_finger"+(f as string)+LR+"MasterINT");
			pFM.wirecolor = internalColor;
			pFM.rotation = bF1.rotation;
			pFM.position = bF1.pos;
	
			pFA = Point pos:ptFinger1.pos isSelected:on centermarker:off axistripod:off cross:off Box:on constantscreensize:off drawontop:off size:(0.5*pointSize);
			pFA.name = (prefix+"_finger"+(f as string)+LR+"AdjusterINT");
			pFA.rotation.controller = Euler_XYZ();
			pFA.wirecolor = internalColor;
			pFA.rotation = bF1.rotation;
			pFA.position = bF1.pos;
	
			-- Parent stuff
			pFM.parent = bHand;		-- parent to Bone which is stuck on end of arm
			pFA.parent = pFM;
			cF1.parent = pFA
			bF1.parent = cF1;
			bFP.parent = bF1;
			zeroOut cF1;
			if (cF2 != undefined) then
				(
				cF2.parent = bF1;		-- control is parented to previous bone
				bF2.parent = cF2;		-- bone is parented to this control
				bFP.parent = bF2;
				zeroOut cF2;
				)
			if (cF3 != undefined) then
				(
				cF3.parent = bF2;
				bF3.parent = cF3;
				bFP.parent = bF3;
				zeroOut cF3;
				)
 

			-- At this point things are all made and linked for the finger...
			-- we need to set up the auto rotations by wiring from the attrs.
		    if (EmptyModifier != undefined) then 
				cntHD = "$"+cHand.name+".Attribute_Holder.Custom_Attributes";
			else
				cntHD = "$"+cHand.name+".Custom_Attributes";
	
--			format "Wiring %...\n" f;
			str = "paramWire.connect "+cntHD+"[#finger"+(f as string)+"] $"+bF1.name+".rotation.controller[3] \"finger"+(f as string)+"/100.0*"+(fDig1Mult as string)+"*"+((degToRad(360)/fingerDigits) as string)+"\";\n";
			if (bF2 != undefined) then 
				str += "paramWire.connect "+cntHD+"[#finger"+(f as string)+"] $"+bF2.name+".rotation.controller[3] \"finger"+(f as string)+"/100.0*"+(fDig2Mult as string)+"*"+((degToRad(360)/fingerDigits) as string)+"\";\n";
			if (bF3 != undefined) then 
				str += "paramWire.connect "+cntHD+"[#finger"+(f as string)+"] $"+bF3.name+".rotation.controller[3] \"finger"+(f as string)+"/100.0*"+(fDig3Mult as string)+"*"+((degToRad(360)/fingerDigits) as string)+"\";\n";
			execute str;			
--			format "%% wired.\n" f LR;
	
			-- Finally take care of spread if needed
		    if (numFingers > 1) then
				(
				deg = 0;
				if (f <= (numFingers/2)) then
					deg = degToRad(-10);
				else
					deg = degToRad(10);
						-- outer fingers do more...
				if (f == 1) then
					deg -= degToRad(20);
				if (f == numFingers) then
					deg += degToRad(20);
--				format "Doing spread %...\n" f;
				str = "paramWire.connect "+cntHD+"[#spread] $'"+pFA.name+"'.rotation.controller[3] \"spread/100.0*"+(deg as string)+"\";\n";
				execute str;
--				format "Spread %% done.\n" f LR;
				)		
	
			)
		else (
			format "WARNING: Can't create Finger %%.  pointHelpers are missing or Finger already Exists.\n" f LR;
			)

		) -- end of numFingers loop

	format "% Fingers created.\n" LR;
    return 1;
)


-- -----------------------------------------------------------------------------------------------

/* 
 * mirrorObjX() - Given the objname to mirror and a newname for the new object,
 *		this creates a mirrored copy on the X axis.
 */
fn mirrorObjX objname newname =
(
	if (objExists(objname) and (objExists(newname) != true)) then
		(
		coordsys world
			(
			newobj = execute("copy "+objname+";")
			newobj.name = newname;
			newobj.position.x = -1 * newobj.position.x;
			scale newobj [-1,1,1];
			newobj.wirecolor = execute(objname+".wirecolor")
			)
		)
)



-- -----------------------------------------------------------------------------------------------


/* 
 * mirrorPtHelpers() - Given a side to mirror, this duplicates and mirrors helpers 
 *		across the other side and names them correctly.
 */
fn mirrorPtHelpers LR =
(
		-- figure which side is the opposite
    if (LR == "L") then
		LR2 = "R";
	else
		LR2 = "L";

	mirrorObjX ("$pt"+LR+"Leg") ("pt"+LR2+"Leg");
	mirrorObjX ("$pt"+LR+"Knee") ("pt"+LR2+"Knee");
	mirrorObjX ("$pt"+LR+"Ankle") ("pt"+LR2+"Ankle");
	mirrorObjX ("$pt"+LR+"Ball") ("pt"+LR2+"Ball");
	mirrorObjX ("$pt"+LR+"Toe") ("pt"+LR2+"Toe");

	mirrorObjX ("$pt"+LR+"2Leg") ("pt"+LR2+"2Leg");
	mirrorObjX ("$pt"+LR+"2Knee") ("pt"+LR2+"2Knee");
	mirrorObjX ("$pt"+LR+"2Ankle") ("pt"+LR2+"2Ankle");
	mirrorObjX ("$pt"+LR+"2Ball") ("pt"+LR2+"2Ball");
	mirrorObjX ("$pt"+LR+"2Toe") ("pt"+LR2+"2Toe");


	mirrorObjX ("$pt"+LR+"Clav") ("pt"+LR2+"Clav");
	mirrorObjX ("$pt"+LR+"UpArm") ("pt"+LR2+"UpArm");
	mirrorObjX ("$pt"+LR+"LoArm") ("pt"+LR2+"LoArm");
	mirrorObjX ("$pt"+LR+"Hand") ("pt"+LR2+"Hand");
	mirrorObjX ("$pt"+LR+"HandTip") ("pt"+LR2+"HandTip");

	mirrorObjX ("$ptThumb"+LR+"1") ("ptThumb"+LR2+"1");
	mirrorObjX ("$ptThumb"+LR+"2") ("ptThumb"+LR2+"2");
	mirrorObjX ("$ptThumb"+LR+"3") ("ptThumb"+LR2+"3");
	mirrorObjX ("$ptThumb"+LR+"4") ("ptThumb"+LR2+"4");
	mirrorObjX ("$ptThumb"+LR+"Up") ("ptThumb"+LR2+"Up");

	mirrorObjX ("$ptFinger1"+LR+"1") ("ptFinger1"+LR2+"1");
	mirrorObjX ("$ptFinger1"+LR+"2") ("ptFinger1"+LR2+"2");
	mirrorObjX ("$ptFinger1"+LR+"3") ("ptFinger1"+LR2+"3");
	mirrorObjX ("$ptFinger1"+LR+"4") ("ptFinger1"+LR2+"4");

	mirrorObjX ("$ptFinger2"+LR+"1") ("ptFinger2"+LR2+"1");
	mirrorObjX ("$ptFinger2"+LR+"2") ("ptFinger2"+LR2+"2");
	mirrorObjX ("$ptFinger2"+LR+"3") ("ptFinger2"+LR2+"3");
	mirrorObjX ("$ptFinger2"+LR+"4") ("ptFinger2"+LR2+"4");

	mirrorObjX ("$ptFinger3"+LR+"1") ("ptFinger3"+LR2+"1");
	mirrorObjX ("$ptFinger3"+LR+"2") ("ptFinger3"+LR2+"2");
	mirrorObjX ("$ptFinger3"+LR+"3") ("ptFinger3"+LR2+"3");
	mirrorObjX ("$ptFinger3"+LR+"4") ("ptFinger3"+LR2+"4");

	mirrorObjX ("$ptFinger4"+LR+"1") ("ptFinger4"+LR2+"1");
	mirrorObjX ("$ptFinger4"+LR+"2") ("ptFinger4"+LR2+"2");
	mirrorObjX ("$ptFinger4"+LR+"3") ("ptFinger4"+LR2+"3");
	mirrorObjX ("$ptFinger4"+LR+"4") ("ptFinger4"+LR2+"4");

	mirrorObjX ("$ptToe1"+LR+"1") ("ptToe1"+LR2+"1");
	mirrorObjX ("$ptToe1"+LR+"2") ("ptToe1"+LR2+"2");
	mirrorObjX ("$ptToe1"+LR+"3") ("ptToe1"+LR2+"3");
	mirrorObjX ("$ptToe1"+LR+"4") ("ptToe1"+LR2+"4");
	mirrorObjX ("$ptToe"+LR+"Up") ("ptToe"+LR2+"Up");

	mirrorObjX ("$ptToe2"+LR+"1") ("ptToe2"+LR2+"1");
	mirrorObjX ("$ptToe2"+LR+"2") ("ptToe2"+LR2+"2");
	mirrorObjX ("$ptToe2"+LR+"3") ("ptToe2"+LR2+"3");
	mirrorObjX ("$ptToe2"+LR+"4") ("ptToe2"+LR2+"4");

	mirrorObjX ("$ptToe3"+LR+"1") ("ptToe3"+LR2+"1");
	mirrorObjX ("$ptToe3"+LR+"2") ("ptToe3"+LR2+"2");
	mirrorObjX ("$ptToe3"+LR+"3") ("ptToe3"+LR2+"3");
	mirrorObjX ("$ptToe3"+LR+"4") ("ptToe3"+LR2+"4");

	mirrorObjX ("$ptToe4"+LR+"1") ("ptToe4"+LR2+"1");
	mirrorObjX ("$ptToe4"+LR+"2") ("ptToe4"+LR2+"2");
	mirrorObjX ("$ptToe4"+LR+"3") ("ptToe4"+LR2+"3");
	mirrorObjX ("$ptToe4"+LR+"4") ("ptToe4"+LR2+"4");

	mirrorObjX ("$ptToe5"+LR+"1") ("ptToe5"+LR2+"1");
	mirrorObjX ("$ptToe5"+LR+"2") ("ptToe5"+LR2+"2");
	mirrorObjX ("$ptToe5"+LR+"3") ("ptToe5"+LR2+"3");
	mirrorObjX ("$ptToe5"+LR+"4") ("ptToe5"+LR2+"4");


	mirrorObjX ("$ptToe1"+LR+"21") ("ptToe1"+LR2+"21");
	mirrorObjX ("$ptToe1"+LR+"22") ("ptToe1"+LR2+"22");
	mirrorObjX ("$ptToe1"+LR+"23") ("ptToe1"+LR2+"23");
	mirrorObjX ("$ptToe1"+LR+"24") ("ptToe1"+LR2+"24");
	mirrorObjX ("$ptToe"+LR+"2Up") ("ptToe"+LR2+"2Up");

	mirrorObjX ("$ptToe2"+LR+"21") ("ptToe2"+LR2+"21");
	mirrorObjX ("$ptToe2"+LR+"22") ("ptToe2"+LR2+"22");
	mirrorObjX ("$ptToe2"+LR+"23") ("ptToe2"+LR2+"23");
	mirrorObjX ("$ptToe2"+LR+"24") ("ptToe2"+LR2+"24");

	mirrorObjX ("$ptToe3"+LR+"21") ("ptToe3"+LR2+"21");
	mirrorObjX ("$ptToe3"+LR+"22") ("ptToe3"+LR2+"22");
	mirrorObjX ("$ptToe3"+LR+"23") ("ptToe3"+LR2+"23");
	mirrorObjX ("$ptToe3"+LR+"24") ("ptToe3"+LR2+"24");

	mirrorObjX ("$ptToe4"+LR+"21") ("ptToe4"+LR2+"21");
	mirrorObjX ("$ptToe4"+LR+"22") ("ptToe4"+LR2+"22");
	mirrorObjX ("$ptToe4"+LR+"23") ("ptToe4"+LR2+"23");
	mirrorObjX ("$ptToe4"+LR+"24") ("ptToe4"+LR2+"24");

	mirrorObjX ("$ptToe5"+LR+"21") ("ptToe5"+LR2+"21");
	mirrorObjX ("$ptToe5"+LR+"22") ("ptToe5"+LR2+"22");
	mirrorObjX ("$ptToe5"+LR+"23") ("ptToe5"+LR2+"23");
	mirrorObjX ("$ptToe5"+LR+"24") ("ptToe5"+LR2+"24");

--  mirrorObjX ("$pt"+LR+"") ("pt"+LR2+"");

)

-- Rollouts --------------------------------------------------------------------------------------

rollout ro_CCcreate "Create"
(
    group "ptHelper Setup" 
		(
	    button b_mirrorL "Mirror L to R" offset:[0,0] width:70 across:2;
	    button b_mirrorR "Mirror R to L" offset:[6,0] width:70;
		)

    group "Creation" 
		(
		edittext et_prefix "Name: " fieldwidth:80 text:"char" width:148;
	    button b_createAll "Create ALL" width:150;
		)

    group "Legs" 
		(
		button b_createRLeg "R Leg" width:73 across:2;
		button b_createLLeg "L Leg" width:73;
		checkbox cb_stretchyLegs "Stretchy Legs" checked:true align:#center;
        button b_createRToes "R Toes" width:73 across:2 ;
        button b_createLToes "L Toes" width:73;
		label lbl_TDig "Toe Digit Rotation %'s:" align:#left;
		spinner spn_TDig1 "" range:[-100,100,tDig1Mult] offset:[-2,0] fieldWidth:40 across:3;
		spinner spn_TDig2 "" range:[-100,100,tDig2Mult] offset:[0,0] fieldWidth:40 ;
		spinner spn_TDig3 "" range:[-100,100,tDig3Mult] offset:[2,0] fieldWidth:40 ;
		)

    group "Base and Body" 
		(
		button b_createHips "Hips" width:150;
		button b_createPlace "Placement & Ground" width:150;
		button b_createSpine "Spine" width:150;
		button b_createHead "Neck & Head" width:150;
		checkbox cb_headIndep "Head is Independent" checked:true align:#center;
		)

	on et_prefix changed str do
		(
			-- store global as name but with spaces as underscores
		prefix = replaceChar str " " "_";
		)
		
    group "Arms" 
		(
		button b_createRArm "R Arm" width:73 across:2;
		button b_createLArm "L Arm" width:73;
		checkbox cb_stretchyArms "Stretchy Arms" checked:true align:#center;
		button b_createRFingers "R Fingers" width:73 across:2;
		button b_createLFingers "L Fingers" width:73;
		label lbl_FDig "Finger Digit Rotation %'s" align:#left;
		spinner spn_FDig1 "" range:[-100,100,fDig1Mult] offset:[-2,0] fieldWidth:40 across:3;
		spinner spn_FDig2 "" range:[-100,100,fDig2Mult] offset:[0,0] fieldWidth:40 ;
		spinner spn_FDig3 "" range:[-100,100,fDig3Mult] offset:[2,0] fieldWidth:40 ;
		)

	group "CTRL Colors"
		(
		colorpicker cp_rightColor "Right:" color:rightColor across:2 fieldwidth:35 height:20;
		colorpicker cp_leftColor "Left:" color:leftColor offset:[13,0] fieldwidth:35 height:20;
		colorpicker cp_bodyColor "Body1:" color:bodyColor offset:[-5,0] across:2 fieldwidth:35 height:20;
		colorpicker cp_bodyColor2 "Body2:" color:bodyColor2 offset:[1,0] fieldwidth:35 height:20;
		colorpicker cp_headColor "Head:" color:headColor offset:[-1,0] across:2 fieldwidth:35 height:20;
		colorpicker cp_baseColor "Base:" color:baseColor offset:[7,0] fieldwidth:35 height:20;
		colorpicker cp_internalColor "Int.:" color:internalColor offset:[10,0] fieldwidth:35 height:20 across:2;
		colorpicker cp_boneColor "Bone:" color:boneColor offset:[6,0] fieldwidth:35 height:20;
		)

    on b_mirrorL pressed do
		(
		undo "charRigger: Mirror L ptHelpers" on
			(
			setWaitCursor();
			mirrorPtHelpers "L";
			setArrowCursor();
			)
		)

    on b_mirrorR pressed do
		(
		undo "charRigger: Mirror R ptHelpers" on
			(
			setWaitCursor();
			mirrorPtHelpers "R";
			setArrowCursor();
			)
		)

    on b_createAll pressed do 
		(
		undo "charRigger: Create All" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createHips();
			createLeg("L");
			createLeg("L2");
			createLeg("R");
			createLeg("R2");
			createToes("L");
			createToes("L2");
			createToes("R");
			createToes("R2");
			createPlacement();
			createSpine();
			createHead();
			createArm("L");
			createArm("R");
			createFingers("L");
			createFingers("R");
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)

    on b_createLLeg pressed do 
		(
		undo "charRigger: Create L Leg" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createHips();
			createLeg("L");
			createLeg("L2");
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)

    on b_createRLeg pressed do 
		(
		undo "charRigger: Create R Leg" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createHips();
			createLeg("R");
			createLeg("R2");
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)

	on cb_stretchyLegs changed state do
		(
		stretchyLegs = state;	-- store in global variable
		) 

	on b_createHips pressed do
		(
		undo "charRigger: Create Hips" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createHips();
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)			

	on b_createPlace pressed do
		(
		undo "charRigger: Create Placement" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createPlacement();
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)

	on b_createSpine pressed do
		(
		undo "charRigger: Create Spine" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createHips();
			createSpine();
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)

	on b_createHead pressed do
		(
		undo "charRigger: Create Head" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createHead();
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)

	on cb_headIndep changed state do
		(
		headIndep = state;	-- store in global variable
		) 


    on b_createLArm pressed do 
		(
		undo "charRigger: Create L Arm" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createArm("L");
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)

    on b_createRArm pressed do 
		(
		undo "charRigger: Create R Arm" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createArm("R");
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)

	on cb_stretchyArms changed state do
		(
		stretchyArms = state;	-- store in global variable
		) 

    on b_createLFingers pressed do 
		(
		undo "charRigger: Create L Fingers" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createFingers("L");
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)

    on b_createRFingers pressed do 
		(
		undo "charRigger: Create R Fingers" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createFingers("R");
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)
	
	on spn_FDig1 changed val do
		(
		fDig1Mult = val;
		)

	on spn_FDig2 changed val do
		(
		fDig2Mult = val;
		)

	on spn_FDig3 changed val do
		(
		fDig3Mult = val;
		)

	on spn_TDig1 changed val do
		(
		tDig1Mult = val;
		)

	on spn_TDig2 changed val do
		(
		tDig2Mult = val;
		)

	on spn_TDig3 changed val do
		(
		tDig3Mult = val;
		)
		
    on b_createLToes pressed do 
		(
		undo "charRigger: Create L Toes" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createToes("L");
			createToes("L2");
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)

    on b_createRToes pressed do 
		(
		undo "charRigger: Create R Toes" on 
			(
			max create mode;	-- this speeds it up for some reason!
			setTimeline rStart rEnd;
			setWaitCursor();
			createToes("R");
			createToes("R2");
			makeNiceSets();
			setArrowCursor();
			setTimeline 0 100;
			)
		)
	on cp_rightColor changed clr do
		(
			rightColor = clr;		-- store in global var
		)

	on cp_leftColor changed clr do
		(
			leftColor = clr;		-- store in global var
		)

	on cp_bodyColor changed clr do
		(
			bodyColor = clr;		-- store in global var
		)

	on cp_bodyColor2 changed clr do
		(
			bodyColor2 = clr;		-- store in global var
		)
		
	on cp_headColor changed clr do
		(
			headColor = clr;		-- store in global var
		)
		
	on cp_baseColor changed clr do
		(
			baseColor = clr;		-- store in global var
		)

	on cp_internalColor changed clr do
		(
			internalColor = clr;		-- store in global var
		)

	on cp_boneColor changed clr do
		(
			boneColor = clr;		-- store in global var
		)

  )  -- end of Create Rollout



rollout ro_CCabout "About"
(
	label abt_lbl1 "charRigger"
	label abt_lbl2 "Version"
	label abt_lbl3 "Built: "
	label abt_lbl4 ""
	label abt_lbl5 "by Michael B. Comet"
	label abt_lbl6 "comet@comet-cartoons.com"
	label abt_lbl7 "www.comet-cartoons.com"
	label abt_lbl7b ""
	label abt_lbl8 "Copyright 2001,2002"
	label abt_lbl9 "Michael B. Comet"
	label abt_lbl10 "All Rights Reserved."
	label abt_lbl12 ""	
	on ro_CCabout open do
	(	
		abt_lbl2.text = ("Version "+cRversion);
		abt_lbl3.text = ("Built: "+cRdate);
	)
	
)


-- Just make this thing a floater...no need for utility

ro_CCfloater = (newRolloutFloater ("charRigger "+cRversion) 200 760)
addRollout ro_CCcreate ro_CCfloater
addRollout ro_CCabout ro_CCfloater rolledUp:true




) -- end of Macroscript


